#R code belonging to the cross feeding analyses of publication "Disentangling metabolic functions of bacteria in the honey bee gut" Kesnerova & Mars et al, PLOS Biology, 2017

#----------------------------------------------------------------------------------------
setwd("/Users/admin/Downloads/Kesnerova_et_al_PB_2017_R_analyses")

#the excel file of interest is bee_second data.xls
excel_filename <- "bee microbiota metabolomics data cross feeding.xlsx" #this is the file merged with the Rutin ion

require(readxl)
injections_mean_supFO <- read_excel(excel_filename, sheet=1) 
ions_mean_supFO <- read_excel(excel_filename, sheet=2) 
annotation_mean_supFO <- read_excel(excel_filename, sheet=3) 
intensities_mean_supFO <- read_excel(excel_filename, sheet=4) 

injections_mean_supFO <-  as.data.frame(injections_mean_supFO, stringsAsFactors=F)
ions_mean_supFO <-  as.data.frame(ions_mean_supFO, stringsAsFactors=F)
annotation_mean_supFO <-  as.data.frame(annotation_mean_supFO, stringsAsFactors=F)
intensities_mean_supFO <-  as.data.frame(intensities_mean_supFO, stringsAsFactors=F)

#---------------------------------------------------
pollen_ions <- read.delim("ions in pollen and substrate group 20170302.tsv", stringsAsFactors=F, sep="\t")
dim(pollen_ions) #number of substrates, 117

#best to match by formula, annotation will be the same
ions_in_pollen <- which(ions_mean_supFO$'Top annotation formula' %in% pollen_ions$Top.annotation.formula)
length(ions_in_pollen) #38
#--------------------------------------------------

#split the dsUser2 column 
head(injections_mean_supFO)
injections_mean_supFO$dsUser2
table(injections_mean_supFO$dsPert)

#all are length 6 now
pert_split_temp <- strsplit(injections_mean_supFO$dsUser2, "_", fixed=TRUE)
pert_split_meta <- as.data.frame(do.call(rbind, pert_split_temp))
pert_split_meta <- pert_split_meta[,1:5]
names(pert_split_meta) <- c("PosNeg", "addition", "time", "replicate", "strain")

#dropping levels
for (i in 1:ncol(pert_split_meta)) {
	pert_split_meta[,i] <- as.character(pert_split_meta[,i])
}

pert_split_meta$comb <- paste(pert_split_meta$addition, pert_split_meta$strain, pert_split_meta$time)
table(pert_split_meta$comb)

pert_split_meta$unique <- paste(pert_split_meta$addition, pert_split_meta$strain)
table(pert_split_meta$unique)

#for conditioning combine the condition, mic and an PosNeg
pert_split_meta$condition <- paste(pert_split_meta$addition, pert_split_meta$PosNeg)
table(pert_split_meta$condition)
pert_split_meta$time <- injections_mean_supFO$dsAmount

#----------------------------------------------------------------------------------------
#do the ttest analyses between extreme time points on all groups. And compute R2 across the log2 Fold changes

conditioning_stats_list <- list()
conditioning_index_list <- list()
for (i in 1:length(unique(pert_split_meta$condition))) {
	pval <- seq(1, 1, length.out=nrow(intensities_mean_supFO))
	log2 <- seq(0, 0, length.out=nrow(intensities_mean_supFO))
	cond_comp <- data.frame(pval, log2)
	cond_comp$start_int <- "-"
	cond_comp$end_int <- "-"
	cond_comp$R2 <- "-"
	cond_comp$cor <- "-"
	cond_comp$start_int_sd <- "-"
	cond_comp$end_int_sd <- "-"
	cond_comp$se_log2 <- "-"
	
	rows <- which(pert_split_meta$condition == unique(pert_split_meta$condition)
	[i])
	start_time <- min(pert_split_meta$time[rows])
	end_time <- max(pert_split_meta$time[rows])
	rows_start <- rows[which(pert_split_meta$time[rows] == start_time)]
	rows_end <- rows[which(pert_split_meta$time[rows] == end_time)]
	conditioning_index_list[[i]] <-  rows
	for (j in 1:nrow(intensities_mean_supFO)) {
		cond_comp$start_int[j] <- round(mean(as.numeric(intensities_mean_supFO[j, 
		rows_start])), digits=0)
  		cond_comp$end_int[j] <- round(mean(as.numeric(intensities_mean_supFO[j, 
  		rows_end])), digits=0)
		cond_comp$start_int_sd[j] <- sd(as.numeric(intensities_mean_supFO[j, 
		rows_start]))
  		cond_comp$end_int_sd[j] <- sd(as.numeric(intensities_mean_supFO[j, 
  		rows_end]))	  
		#need standard errors of both to calculate the fold change error
		x <- log2(as.numeric(intensities_mean_supFO[j, rows_start]))
		x_se <- sqrt(var(x)/length(x))
		y <- log2(as.numeric(intensities_mean_supFO[j, rows_end]))
		y_se <- sqrt(var(y)/length(y))
		cond_comp$se_log2[j] <- sqrt(x_se^2+y_se^2)
		cond_comp$log2[j] <- log2(mean(as.numeric(intensities_mean_supFO[j, 
		rows_end])) / mean(as.numeric(intensities_mean_supFO[j, rows_start])))
 		fit <- unlist(t.test(intensities_mean_supFO[j, rows_start], 
 		intensities_mean_supFO[j, rows_end]))
  		cond_comp$pval[j] <- as.numeric(fit[3])
  		fit_lm <- lm(as.numeric(log2(intensities_mean_supFO[j, rows])) ~ 
  		pert_split_meta$time[rows])
  		cond_comp$R2[j] <- summary(fit_lm)$adj.r.squared
  		cond_comp$cor[j] <- cor(as.numeric(log2(intensities_mean_supFO[j, rows])),
  		 pert_split_meta$time[rows])
	}
	cond_comp$pvalBH <- p.adjust(cond_comp$pval, method="BH")
	#replacing 0 log2 fc with range of 0.1 to 0.01 for plotting
	cond_comp$log2[which(cond_comp$log2 == 0)] <- seq(0.1, 0.01, 
						length.out=length(which(cond_comp$log2 == 0))) 
	conditioning_stats_list[[i]] <- cond_comp
}
names(conditioning_stats_list) <- unique(pert_split_meta$condition)
names(conditioning_index_list) <- unique(pert_split_meta$condition)
length(conditioning_stats_list)

head(conditioning_stats_list[[1]])

#---------------------------------------
#log2 normalize with respect to mean of the first time point

int_log2 <- conditioning_index_list
times_list_log2 <- conditioning_index_list

for (i in 1:length(conditioning_index_list)) {
	rows <- conditioning_index_list[[i]]
  	int_temp <- intensities_mean_supFO[, rows]
 	times_percol <- pert_split_meta$time[rows]
 	int_temp_log2 <- list()
  	time_temp <- c()
  	times_zero_cols <- which(times_percol == "0")
  	int_temp_log2 <- log2(int_temp / apply(int_temp[,times_zero_cols], 1, FUN=mean))
  	int_temp_log2 <- int_temp_log2[,order(times_percol)]
  	int_log2[[i]] <- int_temp_log2
  	times_list_log2[[i]] <- times_percol[order(times_percol)]
}
head(int_log2[[1]])
names(int_log2)

#------------------------------------------------------------------------------------------------------
#pairing the following conditions: 

#microaerobic
cutoff <- 1
names(conditioning_stats_list)[[2]]
names(conditioning_stats_list)[[1]]
increase_cond <- which(conditioning_stats_list[[2]]$log2 >= cutoff)
decrease_growth <- which(conditioning_stats_list[[1]]$log2 <= -cutoff)
ions_exchanged <- intersect(increase_cond, decrease_growth)
ions_mean_supFO$'Top annotation name'[ions_exchanged]

#-----------------------------------------
#plotting

#add error bars function
addErrorBars <- function( x, y, err, ... ){ 	arrows( x, y+err, x, y-err, angle=90, code=3, length=.02, ...) }

require(RColorBrewer)
colors_unique <- brewer.pal(7,"Dark2")[c(6,2,4,3,1,7)]

ion_names <- c()
ion_names_temp <- strsplit(ions_mean_supFO$'Top annotation name', ";", fixed=TRUE)
ion_names <- do.call(rbind, ion_names_temp)
ion_names_first <- ion_names[,1]
ion_nr_ann <- unlist(lapply(strsplit(ions_mean_supFO$'Top annotation ids', ";", fixed=TRUE), FUN=length))
ion_names_first[ion_nr_ann > 1] <- paste(ion_names_first[ion_nr_ann > 1], "*", sep="")

dataset1 <- 2
dataset2 <- 1

pdf("exchanged G S 1 log2 microaerobic_one panel.pdf", width=8, height=2.5)
par(mfrow=c(1, 3))
	rows <- which(pert_split_meta$condition == 
	unique(pert_split_meta$condition)[dataset1])
	par(mar=c(3.5,3.5,2,1))
	times <- times_list_log2[[dataset1]]
	inten <- int_log2[[dataset1]]
	plot(times, inten[1,], las=1, xlab="", ylab="", pch="", col=rgb(0.6,0.6,0.6,0.6), type="n", ylim=c(-.5, 5.5), cex.axis=1.4) 
	mtext("time (hours)", side=1, line=2.5, cex=1)
  	mtext("log2(fold change)", side=2, line=2.2, cex=1)
  	title("G conditioning", line=0.5, cex=1)
	for (i in length(ions_exchanged):1) {	
		ion_to_plot <- ions_exchanged[i]
		fit_temp <- loess.smooth(times, inten[ion_to_plot,], span=2)
		lines(fit_temp$x, fit_temp$y, lwd = 2, lty = 2, col= colors_unique[i])
		temp_sds <- aggregate(as.numeric(inten[ion_to_plot,]), list(times), FUN=sd)$x
		temp <- aggregate(as.numeric(inten[ion_to_plot,]), list(times), FUN=mean)
		temp_time <- temp$Group.1
		temp_means <- temp$x
		addErrorBars(temp_time, temp_means, temp_sds, lwd=1.5, col= colors_unique[i])
	}

  	times <- times_list_log2[[dataset2]]
	inten <- int_log2[[dataset2]]
	par(mar=c(3.5,1.5,2,0))
 	plot(times, inten[ion_to_plot,], las=1, xlab="", ylab="", pch="", 	
 	col=rgb(0.6,0.6,0.6,0.6), type="n", ylim=c(-3.5, 0.5), cex.axis=1.4)  
	mtext("time (hours)", side=1, line=2.5, cex=1)
  	title("S growth", line=0.5, cex=1)
	for (i in length(ions_exchanged):1) {	
		ion_to_plot <- ions_exchanged[i]
		fit_temp <- loess.smooth(times, inten[ion_to_plot,], span=1)
		lines(fit_temp$x, fit_temp$y, lwd = 2, lty = 1, col= colors_unique[i])
		temp_sds <- aggregate(as.numeric(inten[ion_to_plot,]), list(times), FUN=sd)$x
		temp <- aggregate(as.numeric(inten[ion_to_plot,]), list(times), FUN=mean)
		temp_time <- temp$Group.1
		temp_means <- temp$x
		addErrorBars(temp_time, temp_means, temp_sds, lwd=1.5, col= colors_unique[i])
	}
	
	mz <- substr(ions_mean_supFO$ionMz, 1,7)
	names_legend <- paste(mz[ions_exchanged], "   ",   	ion_names_first[ions_exchanged])
  	par(mar=c(1,1,1,1))		
	plot(1:10, type="n", axes=F, xlab="", ylab="")
	legend(0, 10, legend=names_legend, pch="", text.col=colors_unique, cex=1, bty="n")		
dev.off()
