#R code belonging to the supernatant analyses of publication "Disentangling metabolic functions of bacteria in the honey bee gut" Kesnerova & Mars et al, PLOS Biology, 2017

#----------------------------------------------------------------------------------------
setwd("/Users/admin/Downloads/Kesnerova_et_al_PB_2017_R_analyses")

excel_filename <- "bee microbiota metabolomics data in vitro.xlsx" 

require(readxl)
ions_sup <- read_excel(excel_filename, sheet=1) 
annotation_sup <- read_excel(excel_filename, sheet=2) 
injections_mean_sup <- read_excel(excel_filename, sheet=3) 
intensities_mean_sup <- read_excel(excel_filename, sheet=4) 

injections_mean_sup <-  as.data.frame(injections_mean_sup, stringsAsFactors=F)
ions_sup <-  as.data.frame(ions_sup, stringsAsFactors=F)
annotation_sup <-  as.data.frame(annotation_sup, stringsAsFactors=F)
intensities_mean_sup <-  as.data.frame(intensities_mean_sup, stringsAsFactors=F)

#---------------------------------------------------
#matching the annotated ions by their chemical formula

pollen_ions <- read.delim("ions in pollen and substrate group 20170302.tsv", stringsAsFactors=F, sep="\t")
dim(pollen_ions) #number of substrates, 117

#best to match by formula, annotation will be the same
ions_in_pollen <- which(ions_sup$'Top annotation formula' %in% pollen_ions$Top.annotation.formula)
length(ions_in_pollen) #60
#--------------------------------------------------
#split MRS, M9, and the species. 

head(injections_mean_sup)
table(injections_mean_sup$dsPert)

injections_mean_sup$dsUser2cor <- gsub("NegMRS_", "", injections_mean_sup$dsUser2)
injections_mean_sup$dsUser2cor <- gsub("NegM9_", "", injections_mean_sup$dsUser2cor)
injections_mean_sup$dsUser2cor <- gsub("142_", "", injections_mean_sup$dsUser2cor)
#all are length 6 now
pert_split_temp <- strsplit(injections_mean_sup$dsUser2cor, "_", fixed=TRUE)
pert_split_meta <- as.data.frame(do.call(rbind, pert_split_temp))
names(pert_split_meta) <- c("species", "time", "replicate", "mediumOR", "red", "Pollen")
pert_split_meta$comb <- paste(pert_split_meta$species, pert_split_meta$Pollen ,injections_mean$dsAmount)
pert_split_meta$unique <- paste(pert_split_meta$species, pert_split_meta$Pollen)

table(pert_split_meta$comb)
table(pert_split_meta$unique)
#------------------------------------------------------------------------------------------------------
#log2 FC compared to the 0 time point, and p-value

cond_stats_list <- list()
for (i in 1:length(unique(pert_split_meta$unique))) {
	pval <- seq(1, 1, length.out=nrow(intensities_mean_sup))
	log2 <- seq(0, 0, length.out=nrow(intensities_mean_sup))
	cond_comp <- data.frame(pval, log2)
	cond_comp$T0_int <- "-"
	cond_comp$T16_int <- "-"

	rows <- which(pert_split_meta$unique == unique(pert_split_meta$unique)[i])
	rows_T0 <- rows[which(injections_mean_sup$dsAmount[rows] == "0")]
	rows_T16 <- rows[which(injections_mean_sup$dsAmount[rows] == "16")]
	for (j in 1:nrow(intensities_mean_sup)) {
		cond_comp$T0_int[j] <- round(mean(as.numeric(intensities_mean_sup[j, rows_T0])),
		 digits=0)
  		cond_comp$T16_int[j] <- round(mean(as.numeric(intensities_mean_sup[j, rows_T16])),
  		 digits=0)
  		cond_comp$log2[j] <- log2(mean(as.numeric(intensities_mean_sup[j, rows_T16])) / 
  		mean(as.numeric(intensities_mean_sup[j, rows_T0])))
 		fit <- unlist(t.test(intensities_mean_sup[j, rows_T0], intensities_mean_sup[j, 
 		rows_T16]))
  		cond_comp$pval[j] <- as.numeric(fit[3])
	}
	cond_comp$pvalBH <- p.adjust(cond_comp$pval, method="BH")
	#replacing 0 log2 fc with range of 0.1 to 0.01 for plotting
	cond_comp$log2[which(cond_comp$log2 == 0)] <- seq(0.1, 0.01, 
						length.out=length(which(cond_comp$log2 == 0))) 
	cond_stats_list[[i]] <- cond_comp
}
names(cond_stats_list) <- unique(pert_split_meta$unique)
length(cond_stats_list)

#---------------------------------------------
#volcano plots and selecting

#what is the max, to set ylim
unlist(lapply(cond_stats_list, FUN=function(x) {max(abs(x$log2))})) 
max(unlist(lapply(cond_stats_list, FUN=function(x) {max(abs(x$log2))}))) #6.75 is the most extreme value

#only to inspect the data

#pdf("bee microbiota in vitro patterns.pdf", width=12, height=10)
par(mfrow=c(4,5))
for (i in 1:length(cond_stats_list)) {
	cond_stats_data <- cond_stats_list[[i]]	
	par(mar=c(3.5,3.5,1,1))
	plot(cond_stats_data$log2, -log10(cond_stats_data$pvalBH), 
        ylab="", xlab="", las=1, pch=16, col=rgb(0.6, 0.6, 0.6, 0.5), xlim=c(-7,7))
	abline(v=-1, col=rgb(0.6, 0.6, 0.6, 0.5), lwd=2) #1 log2 FC
	abline(v=1, col=rgb(0.6, 0.6, 0.6, 0.5), lwd=2) #1 log2 FC
	abline(h=2, col=rgb(0.6, 0.6, 0.6, 0.5), lwd=2) #at 0.01 / 2 	
	#plot pollen substrates in black
	points(cond_stats_data$log2[ions_in_pollen], -log10(cond_stats_data$pvalBH[ions_in_pollen]), pch=16, col="black")
	title(names(cond_stats_list)[i])
}
#dev.off()
#---------------------------------------------
#find the pollen substate ions that go down with more than 1 log2 FC

cond_allsubs_list <- list()
cond_pollen_list <- list()
for (i in 1:length(cond_stats_list)) {
	cond_stats_data <- cond_stats_list[[i]]
	sig_ions <- which(cond_stats_data$log2 <= -1 & cond_stats_data$pvalBH <= 0.01)
	cond_allsubs_list[[i]] <- sig_ions
	cond_pollen_list[[i]] <- intersect(sig_ions, ions_in_pollen)
}
names(cond_allsubs_list) <- names(cond_stats_list)
names(cond_pollen_list) <- names(cond_stats_list)

df_pol_subs <- as.data.frame(cbind(names(cond_stats_list), unlist(lapply(cond_pollen_list, FUN=length))))

#filter out the ions that also go down in medium
strains <- do.call(rbind, strsplit(names(cond_pollen_list), " "))[,1]

cond_pollen_list_strain <- list()
cond_allsubs_list_strain <- list()
for (i in 1:length(unique(strains))) {
	ind <- which(strains == unique(strains)[i])
	if (length(ind) > 1) {
		ind_PE <- ind[grep("PE", names(cond_pollen_list)[ind])]
		ind_no <- setdiff(ind, ind_PE)
		cond_pollen_list_strain[[i]] <- setdiff(cond_pollen_list[[ind_PE]], 
		cond_pollen_list[[ind_no]]) #first put the PE one
		cond_allsubs_list_strain[[i]] <- setdiff(cond_allsubs_list[[ind_PE]], 
		cond_allsubs_list[[ind_no]]) 		
	}
}
names(cond_pollen_list_strain) <- unique(strains)
names(cond_allsubs_list_strain) <- unique(strains)

unique(unlist(cond_pollen_list_strain))
ions_sup$'Top annotation name'[unique(unlist(cond_pollen_list_strain))]
#864 is quercitrin, 954 is Rutin
names(cond_pollen_list_strain)[grep("864", cond_pollen_list_strain)]
names(cond_pollen_list_strain)[grep("954", cond_pollen_list_strain)]


#get the formulas for the changes and go back to see if these are observed in vivo as well
cond_pollen_inVitro_list <- list()
cond_allsubs_inVitro_list <- list()
for (i in 1:length(cond_pollen_list_strain)) {
	cond_pollen_inVitro_list[[i]] <- ions_sup$'Top annotation formula'[cond_pollen_list_strain[[i]]]
	cond_allsubs_inVitro_list[[i]] <- ions_sup$'Top annotation formula'[cond_allsubs_list_strain[[i]]]

}
names(cond_pollen_inVitro_list) <- names(cond_pollen_list_strain)
save(cond_pollen_inVitro_list, file="cond_pollen_inVitro_list")

names(cond_allsubs_inVitro_list) <- names(cond_pollen_list_strain)
save(cond_allsubs_inVitro_list, file="cond_allsubs_inVitro_list")


#---------------------------------------------
#look for production of compounds on pollen extract

#look for production
cond_pollen_prod_list <- list()
for (i in 1:length(cond_stats_list)) {
	cond_stats_data <- cond_stats_list[[i]]
	cond_pollen_prod_list[[i]] <- which(cond_stats_data$log2 >= 1 & cond_stats_data$pvalBH <= 0.01)
}
names(cond_pollen_prod_list) <- names(cond_stats_list)

ions_sup$'Top annotation name'[cond_pollen_prod_list[[6]]]
cond_pollen_prod_list[[6]][c(7,12)]

cond_pollen_inVitro_prod_list <- list()
for (i in 1:length(cond_pollen_prod_list)) {
	cond_pollen_inVitro_prod_list[[i]] <- ions_sup$'Top annotation formula'[cond_pollen_prod_list[[i]]]
}
names(cond_pollen_inVitro_prod_list) <- names(cond_pollen_prod_list)
save(cond_pollen_inVitro_prod_list, file="cond_pollen_inVitro_prod_list")
#---------------------------------------------

cond_pollen_subs_list_strain_sub <- cond_pollen_list_strain[order(names(cond_pollen_list_strain))][c(1:7,11)]

cond_pollen_prod_list_sel <- cond_pollen_prod_list[grep("PE", names(cond_pollen_prod_list))]
names(cond_pollen_prod_list_sel) <- gsub(" PE", "", names(cond_pollen_prod_list_sel)) 
cond_pollen_prod_list_sel_sub <- cond_pollen_prod_list_sel[order(names(cond_pollen_prod_list_sel))][c(1:8,10)]


#again read in ions_sup
ions_sup <- read_excel(excel_filename, sheet=1) 

colnames_subs <- paste(names(cond_pollen_subs_list_strain_sub), "substr", sep="_")
for (i in 1:length(cond_pollen_subs_list_strain_sub)) {
	ions_sup <- cbind(ions_sup, 0)
	names(ions_sup)[length(ions_sup)] <- colnames_subs[i]
	row <- as.character(cond_pollen_subs_list_strain_sub[[i]])
	if (length(row) > 0) {
	ions_sup[row,length(ions_sup)] <- 1 }
}
colnames_prods <- paste(names(cond_pollen_prod_list_sel_sub), "prod", sep="_")
for (i in 1:length(cond_pollen_prod_list_sel_sub)) {
	ions_sup <- cbind(ions_sup, 0)
	names(ions_sup)[length(ions_sup)] <- colnames_prods[i]
	row <- as.character(cond_pollen_prod_list_sel_sub[[i]])
	if (length(row) > 0) {
	ions_sup[row,length(ions_sup)] <- 1 }
}
#-----------------------------------------
#now add the fold changes of everything to here

rows_PE <- grep("PE", names(cond_stats_list))[c(1,3,4:8)] #manually removed PE PE, A2, and G1
names(cond_stats_list)[rows_PE] #checking
cond_stats_list_sel <- cond_stats_list[rows_PE]
names(cond_stats_list_sel) <- gsub(" PE", "", names(cond_stats_list_sel)) 
cond_stats_list_sel <- cond_stats_list_sel[order(names(cond_stats_list_sel))] 

colnames_log2 <- paste(names(cond_stats_list_sel), "log2FC", sep="_")
colnames_pvalBH <- paste(names(cond_stats_list_sel), "pvalBH", sep="_")

for (i in 1:length(cond_stats_list_sel)) {
	ions_sup <- cbind(ions_sup, 0)
	names(ions_sup)[length(ions_sup)] <- colnames_log2[i]
	ions_sup[length(ions_sup)] <- cond_stats_list_sel[[i]]$log2
	ions_sup <- cbind(ions_sup, 0)
	names(ions_sup)[length(ions_sup)] <- colnames_pvalBH[i]
	ions_sup[length(ions_sup)] <- cond_stats_list_sel[[i]]$pvalBH
}

ions_sup$ions_in_pollen <- "-"
ions_sup$ions_in_pollen[ions_in_pollen] <- "x"

#write.table(ions_sup, "ions in vitro results 2 with S.tsv", row.names=F, quote=FALSE, sep='\t')
#-----------------------------------------
#add compound class to annotation and ions

class <- read.csv("kegg_category_Feb2017.csv", stringsAsFactors=F, sep=",")
head(class)
annotation_sup$detailed_compound_class <- "-"
for (i in 1:nrow(annotation_sup)) {
  id <- which(class$kegg_id == annotation_sup $kegg_id[i]) 
  if (length(id) > 0) {
    annotation_sup$detailed_compound_class[i] <- class$old_kegg_class[id]
  }
}

#all ions_sup
selrows <- list()
for (i in 1:length(ions_sup$ionIdx)) {
  rows <- which(annotation_sup$ionIdx == ions_sup$ionIdx[i])
  selrows[[i]] <- rows[1]
}
gut_ions_ann <- unlist(selrows)

ions_sup$compound_class <- annotation_sup$detailed_compound_class[gut_ions_ann]

#write.table(ions_sup, "ions in vitro results v3 with compound class.tsv", row.names=F, quote=FALSE, sep='\t')
#write.table(annotation_sup, "annotations in vitro results compound class.tsv", row.names=F, quote=FALSE, sep='\t')


#-------------------------------------------------------------------------------------
#read in file with annotations to use for plotting volcano plots

df_to_plot <- read_excel("Dataset3_invitro_plotting_1.xlsx", sheet=1) 
head(df_to_plot,1)
#checking stuff
df_to_plot[df_to_plot$plotting_categories == "Glycosylated-flavonoid",]

table(df_to_plot$plotting_categories)
#remove nucleoside stuff, 20170601
df_to_plot$plotting_categories[df_to_plot$plotting_categories == "Nucleoside"] <- "-"

names(df_to_plot)
df_to_plot$'Top-annotation-name'[df_to_plot$plotting_categories == "Flavonoid-breakdown"]
grep("Ascorbate", df_to_plot$'Top-annotation-name')
df_to_plot$plotting_categories[216] <- "Flavonoid-breakdown"

df_to_plot$'Top-annotation-name'[which(df_to_plot$plotting_categories == "Flavonoid-breakdown")] #products

df_to_plot$plotting_categories[intersect(which(df_to_plot$ions_in_pollen == "x"), which(df_to_plot$plotting_categories == "-"))] <- "in pollen"

require(RColorBrewer)
#num_of_colors <- length(unique(df_to_plot$plotting_categories)) -1
#[1] "-"                         "in pollen"                
#[3] "Flavonoid-breakdown"       "Nucleoside"               
#[5] "Nonglycosylated-flavonoid" "Glycosylated-flavonoid"  

color_temp <- brewer.pal(6,"Dark2")
colors_unique <- c("black", color_temp[3], color_temp[2], color_temp[1], color_temp[3])
require(rafalib)
grey_ions <- which(df_to_plot$plotting_categories == "-")
group <- as.fumeric(df_to_plot$plotting_categories[-grey_ions])
colvec <- rep(colors_unique)[group]
categories <- unique(df_to_plot$plotting_categories)


#part of figure 5, manually combined and names changed / added
pdf("bee microbiota in vitro patterns categories cutoff colored.pdf", width=10, height=5)

par(mfrow=c(2,4))
for (i in 1:8) {
	if (i == 8) {
	par(mar=c(1,1,1,1))		
	plot(1:10, type="n", axes=F, xlab="", ylab="")
	legend(1, 10, legend=categories , pch=16, col= c(rgb(0.6, 0.6, 0.6, 0.5), 
	colors_unique), cex=1.2, bty="n")	
	} else {
	par(mar=c(3.5,5,1.5,1))
	cond_stats_data <- cond_stats_list[[rows_PE[i]]]	
	plot(cond_stats_data$log2[grey_ions], -log10(cond_stats_data$pvalBH)[grey_ions], ylab="", xlab="", las=1, pch=16, col=rgb(0.6, 0.6, 0.6, 0.5), xlim=c(-7,7), ylim=c(0,7))
	abline(v=-1, col=rgb(0.6, 0.6, 0.6, 0.5), lwd=1.5) #1 log2 FC
	abline(v=1, col=rgb(0.6, 0.6, 0.6, 0.5), lwd=1.5) #1 log2 FC
	abline(h=2, col=rgb(0.6, 0.6, 0.6, 0.5), lwd=1.5) #at 0.01 / 2 	
	#plot all pollen
	in_pollen <- which(df_to_plot$ions_in_pollen == "x")
	points(cond_stats_data$log2[in_pollen], -log10(cond_stats_data$pvalBH[in_pollen]),
	pch=16, col="black")	
	#overlap with things that show more than 1 fold change
	changing_rows <- which(abs(cond_stats_data$log2) >= 1)
	colored_rows <- c(1:nrow(cond_stats_data))[-grey_ions]
	to_plot <- colored_rows[which(colored_rows %in% changing_rows)]
	points(cond_stats_data$log2[to_plot], -log10(cond_stats_data$pvalBH[to_plot]),
	pch=16, col= colvec[which(colored_rows %in% changing_rows)])	
#	points(cond_stats_data$log2[851], -log10(cond_stats_data$pvalBH[851]), pch=16, col= "red")
	title(names(cond_stats_list)[rows_PE[i]])
	mtext("log2(fold change)", side=1, line=2.2, cex=0.75)
	mtext("-log10(p-value)", side=2, line=2.2, cex=0.75)
	}
}
dev.off()
#-------------------------------------------------------------------------------------

#now go back to the in vivo file if you want to produce the files required to run the rest

#-------------------------------------------------------------------------------------
#in vivo in vitro matching and volcano plot

#adapted to do on all substrates
overlap_in_vivo_in_vitro_substrate  <-read.csv( "overlap_in_vivo_in_vitro_all_substrates.csv", stringsAsFactors=F)
overlap_in_vivo_in_vitro_prod <- read.csv("overlap_in_vivo_in_vitro_products_v2.csv", stringsAsFactors =F)

overlap_idx <- c(overlap_in_vivo_in_vitro_substrate$ionIdx, overlap_in_vivo_in_vitro_prod$ionIdx)
overlap_strain <- c(overlap_in_vivo_in_vitro_substrate$strain, overlap_in_vivo_in_vitro_prod$strain)
overlap_df <- as.data.frame(cbind(as.character(overlap_idx), as.character(overlap_strain)))
names(overlap_df) <- c("ionIdx", "strain")
overlap_df$ionIdx <- as.character(overlap_df$ionIdx)
overlap_df$strain <- as.character(overlap_df$strain)

#match these with the ions_sup file via unique formula, read in the ions file with in vivo data
excel_filename <- "bee microbiota metabolomics data in vivo.xlsx"
require(readxl)
ions <- read_excel(excel_filename, sheet=2) 

overlap_df$ionIdx_sup <- "-"
for (i in 1:nrow(overlap_df)) {
	ionIdx_temp <- as.numeric(overlap_df$ionIdx[i])
	ionIdx_sup_temp <- which(ions_sup$'Top annotation formula' == ions$'Top annotation formula'[ionIdx_temp])
	if (length(ionIdx_sup_temp) > 0) {
		overlap_df$ionIdx_sup[i] <- ionIdx_sup_temp
	}
}

#----------------------------------------------------------------------------------------
#in vivo and in vitro explained volcano plot. Names are adapted manually later.

ion_names <- c()
ion_names_temp <- strsplit(ions_sup$'Top annotation name', ";", fixed=TRUE)
ion_names <- do.call(rbind, ion_names_temp)
ion_names_first <- ion_names[,1]
ion_nr_ann <- unlist(lapply(strsplit(ions_sup$'Top annotation ids', ";", fixed=TRUE), FUN=length))
ion_names_first[ion_nr_ann > 1] <- paste(ion_names_first[ion_nr_ann > 1], "*", sep="")


grey_ions <- which(df_to_plot$plotting_categories == "-")
pdf("bee microbiota in vivo in vitro explained volcano names.pdf", width=10, height=5)
par(mfrow=c(2,4))
for (i in 1:7) {
	par(mar=c(3.5,5,1.5,1))
	cond_stats_data <- cond_stats_list[[rows_PE[i]]]	
	plot(cond_stats_data$log2[grey_ions], -log10(cond_stats_data$pvalBH)[grey_ions], ylab="", xlab="", las=1, pch=16, col=rgb(0.6, 0.6, 0.6, 0.5), xlim=c(-7,7), ylim=c(0,7))
	abline(v=-1, col=rgb(0.6, 0.6, 0.6, 0.5), lwd=1.5) #1 log2 FC
	abline(v=1, col=rgb(0.6, 0.6, 0.6, 0.5), lwd=1.5) #1 log2 FC
	abline(h=2, col=rgb(0.6, 0.6, 0.6, 0.5), lwd=1.5) #at 0.01 / 2 	
	#plot all pollen
	in_pollen <- which(df_to_plot$ions_in_pollen == "x")
	points(cond_stats_data$log2[in_pollen], -log10(cond_stats_data$pvalBH[in_pollen]),
	pch=16, col="black")	
	cond_name_temp <- strsplit(names(cond_stats_list)[rows_PE[i]], " ")[[1]][1]
	row_temp <- which(overlap_df$strain == cond_name_temp)
	if (length(row_temp) > 0) {
		to_plot <- as.numeric(overlap_df$ionIdx_sup[row_temp])
		points(cond_stats_data$log2[to_plot], -log10(cond_stats_data$pvalBH[to_plot]),
	pch=16, col="red")
		text(cond_stats_data$log2[to_plot], -log10(cond_stats_data$pvalBH[to_plot]), labels=ion_names_first[to_plot])	
	}
	title(names(cond_stats_list)[rows_PE[i]])
	mtext("log2(fold change)", side=1, line=2.2, cex=0.75)
	mtext("-log10(p-value)", side=2, line=2.2, cex=0.75)
}
dev.off()


