#R code belonging to the in vivo part of publication "Disentangling metabolic functions of bacteria in the honey bee gut" Kesnerova & Mars et al, PLOS Biology, 2017

#---------------------------------------------------------------------------------------
# the first names were used initially, and are still used for parts of the analyses
#G: 	Ga	Gilliamella apiocola ("Gamma")
#Fp: 	Fp	Frischella perrara ("Gamma")
#S: 	Sa	Snodgrasella alvi (Beta-proteobacteria)
#F5: 	F5	Lactobacillus apis and others ("Firm5")
#F4: 	F4	Lactobacillus mellis/Lactobacillus mellifer ("Firm4")
#B: 	Bi	Bifidobacterium asteroides and others ("Bifido")
#A1: 	Ba	Bartonella apis ("Alpha 1") 
#MF:	MD	Microbiota depleted
#CL:	CL	Colonized

#---------------------------------------------------------------------------------------
#installing required packages
source('http://bioconductor.org/biocLite.R')
biocLite('preprocessCore')
#load package
library(preprocessCore)

install.packages("readxl")
install.packages("rafalib")
install.packages("gplots")
install.packages('VennDiagram')
install.packages("xlsx") #in case you do not have this one

library(readxl)
library(VennDiagram)
library(RColorBrewer)
library(rafalib)
library(xlsx)

#to print colors
display.brewer.all(colorblindFriendly=T)
#------------------------------------------------------------------------------------------------------
#make sure your locale settings are as follows by running:
Sys.setlocale("LC_COLLATE", "en_US.UTF-8")
Sys.setlocale("LC_CTYPE", "en_US.UTF-8")
#------------------------------------------------------------------------------------------------------
#set working directory
#on mac
setwd("/Users/admin/Downloads/Kesnerova_et_al_PB_2017_R_analyses")

#the excel file with the raw data of the in vivo work s bee_second data.xls
excel_filename <- "bee microbiota metabolomics data in vivo.xlsx"

require(readxl) #optionally run "solve readxl issue.R" if data is not imported properly
injections_mean <- read_excel(excel_filename, sheet=1) 
ions <- read_excel(excel_filename, sheet=2) 
annotation <- read_excel(excel_filename, sheet=3) 
intensities_mean <- read_excel(excel_filename, sheet=4) 

dim(injections_mean)
dim(intensities_mean)
if (ncol(intensities_mean) != nrow(injections_mean)) {
	print("something goes wrong with importing .xlsx file")
	print("perhaps you have to run the solve readxl issue script") 
}
head(injections_mean)

injections_mean <-  as.data.frame(injections_mean, stringsAsFactors=F)
ions <-  as.data.frame(ions, stringsAsFactors=F)
annotation <-  as.data.frame(annotation, stringsAsFactors=F)
intensities_mean <-  as.data.frame(intensities_mean, stringsAsFactors=F)

#------------------------------------------
#updating names

ion_names_updated <- read_excel("170520_Reannotation_372ionChanges_Philipp_RM.xlsx", sheet=1) 
head(ion_names_updated)
#ion_names_updated$'Final primary annotation'
sign_ions_pca <- ion_names_updated$ionIdx

#add it as an extra column to the ions file
ions$curated_annotation_sign <- "-"
ions$curated_annotation_sign[ion_names_updated$ionIdx] <- ion_names_updated$'Final primary annotation'

ion_names <- c()
ion_names_temp <- strsplit(ions$'Top annotation name', ";", fixed=TRUE)
ion_names <- do.call(rbind, ion_names_temp)
ion_names_first <- ion_names[,1]
ions$curated_annotation_with_all <- ion_names_first
bool_temp <-ions$curated_annotation_sign == "-"
ions$curated_annotation_with_all[which(bool_temp == FALSE)] <- ions$curated_annotation_sign[which(bool_temp == FALSE)]

#------------------------------------------
#split the data of the different seasons
col_fall <- which(injections_mean$season == "fall")
col_spring <- which(injections_mean$season == "spring")
#----------------------------------------
#looking at number of alternative annotations

nrow(ions) #1079
nrow(annotation) #3270
table(table(annotation$ionIdx))
#  1   2   3   4   5   6   7   8   9  10  11  12  13  14  15  16  17  18 
#518 204  93  74  40  31  32  22  11  13   3  10   2   4   3   1   3   1 

# 19  20  21  22  25  26  27  30  35  36  40 
 # 1   1   1   2   1   1   1   2   1   2   1 
median(table(annotation$ionIdx)) #2
mean(table(annotation$ionIdx)) # 3.03

#table(annotation$ionIdx)
#----------------------------------------
#PCA plot with different versions for the resubmission

ind <- grep("MF|CL|hive fall", injections_mean$full_sample_group)
names_pca <- injections_mean$full_sample_group[ind]
names_pca <- gsub("Gut ", "", names_pca)
names_pca <- gsub("hive fall", "hive", names_pca)
names_pca <- gsub("spring", "exp1", names_pca)
names_pca <- gsub("fall", "exp2", names_pca)
unique(injections_mean$full_sample_group[ind])
intensities_mean_sub <- intensities_mean[,ind] 
names(intensities_mean_sub) <- names_pca

require(RColorBrewer)
colors_unique <- brewer.pal(10, "Paired")[c(1,5,6,2,10)] #for 5 groups
require(rafalib)
group <- as.fumeric(names(intensities_mean_sub))
sort_names <- order(c(unique(names(intensities_mean_sub))))

test_mat1 <- log2(intensities_mean_sub)
#using the covariance matrix first, with scaling
pca = prcomp(test_mat1, center=T, scale=T)
summary(pca)

#used this in first submission & final resubmission

pdf("pca four types.pdf", width=6, height=6)
par(mfrow=c(2,2))
#1
par(mar=c(4,4,2.5,1))
plot(pca$rotation[,1], pca$rotation[,2], col=rep(colors_unique)[group], xlab="PC1 90.2%", ylab="PC2 5.4%", cex=1.2, pch=16,
     cex.axis=1.2, cex.lab=1.2, las=1, ylim=c(-0.25, 0.25), xlim=c(0.140, 0.155))
legend(0.140, 0.27, legend=c(unique(names(intensities_mean_sub)))[sort_names], pch=16, col=colors_unique[sort_names], cex=1.2, bty="n")
title("Metabolome PCA ann. ions, log2", cex.main=0.9)

pca_data <- as.data.frame(rownames(pca$rotation))
pca_data <- cbind(pca_data, pca$rotation[,1], pca$rotation[,2])
names(pca_data) <- c("group", "all_log2_PC1", "all_log2_PC2")

#2
#transform to ion Z-scores
intensities_gut_zscore <- intensities_mean_sub #to get proper dimensions
for (i in 1:nrow(intensities_gut_zscore)) {
  	intensities_gut_zscore[i,] <- (intensities_mean_sub[i,] - mean(as.numeric(intensities_mean_sub[i,]))) / sd(as.numeric(intensities_mean_sub[i,]))
}
pca = prcomp(intensities_gut_zscore, center=T, scale=T)
summary(pca)
par(mar=c(4,4,2.5,1))
plot(pca$rotation[,1], pca$rotation[,2], col=rep(colors_unique)[group], xlab="PC1 36.2%", ylab="PC2 11.1%", cex=1.2, pch=16, cex.axis=1.2, cex.lab=1.2, las=1)
title("Metabolome PCA ann. ions, Z-score", cex.main=0.9)

pca_data <- cbind(pca_data, pca$rotation[,1], pca$rotation[,2])
names(pca_data)[4:5] <- c("all_Zscore_PC1", "all_Zscore_PC2")


#3
test_mat1 <- log2(intensities_mean_sub)
test_mat2 <- test_mat1[sign_ions_pca,]
#using the covariance matrix first, with scaling
pca = prcomp(test_mat2, center=T, scale=T)
par(mar=c(4,4,2.5,1))
plot(pca$rotation[,1], pca$rotation[,2], col=rep(colors_unique)[group], xlab="PC1 86.3%", ylab="PC2 10.0%", cex=1.2, pch=16,
     cex.axis=1.2, cex.lab=1.2, las=1, ylim=c(-0.25, 0.25), xlim=c(0.140, 0.155))
title("Metabolome PCA sig. ions, log2", cex.main=0.9)

pca_data <- cbind(pca_data, pca$rotation[,1], pca$rotation[,2])
names(pca_data)[6:7] <- c("sign_log2_PC1", "sign_log2_PC2")


#4
intensities_gut_zscore <- intensities_mean_sub[sign_ions_pca,] #to get proper dimensions
for (i in 1:nrow(intensities_gut_zscore)) {
  	intensities_gut_zscore[i,] <- (intensities_mean_sub[i,] - mean(as.numeric(intensities_mean_sub[i,]))) / sd(as.numeric(intensities_mean_sub[i,]))
}
pca = prcomp(intensities_gut_zscore, center=T, scale=T)
summary(pca)
par(mar=c(4,4,2.5,1))
plot(pca$rotation[,1], pca$rotation[,2], col=rep(colors_unique)[group], xlab="PC1 38.0%", ylab="PC2 11.5%", cex=1.2, pch=16, cex.axis=1.2, cex.lab=1.2, las=1)
title("Metabolome PCA sig. ions, Z-score", cex.main=0.9)

pca_data <- cbind(pca_data, pca$rotation[,1], pca$rotation[,2])
names(pca_data)[8:9] <- c("sign_Zscore_PC1", "sign_Zscore_PC2")

dev.off()

#write.csv(pca_data, "pca_data.csv", row.names=F)

#--------------------------------------------------------------------------------------
#Welch t-tests on intensities of Gut spring (exp1) and fall (exp2) CL : MF & compare overlap 

ind <- grep("MF|CL", injections_mean$full_sample_group)
intensities_mean_sel <- intensities_mean[,ind] 
injections_mean_sel <- injections_mean[ind,]
names(intensities_mean_sel) <- injections_mean$full_sample_group[ind]
table(names(intensities_mean_sel))

# Gut CL fall Gut CL spring   Gut MF fall Gut MF spring 
#           12             6            13             8 
#----------------------------------------

seasons <- unique(injections_mean_sel$season)
ttest_res_list <- list()
for (i in 1:3) {
	season_cols <- which(injections_mean_sel$season == seasons[i])
	if (i == 3) { season_cols <- 1:length(injections_mean_sel$season) }
	rows_test <- season_cols[which(injections_mean_sel$group[season_cols] == "CL")]
	rows_ctrl <- season_cols[which(injections_mean_sel$group[season_cols] == "MF")]
	
	pval <- seq(1, 1, length.out=nrow(intensities_mean_sel))
	log2 <- seq(0, 0, length.out=nrow(intensities_mean_sel))
	comp <- data.frame(pval, log2)
	comp$se_log2 <- 0
	comp$ctr_int <- 0
	comp$test_int <- 0
	comp$ctr_int_sd <- 0
	comp$test_int_sd <- 0
	for (j in 1:nrow(intensities_mean_sel)) {
 		comp$ctr_int[j] <- round(mean(as.numeric(intensities_mean_sel[j, 
 		rows_test])), digits=0)
  		comp$test_int[j] <- round(mean(as.numeric(intensities_mean_sel[j, 
  		rows_ctrl])), digits=0)  		  	
		comp$ctr_int_sd[j] <- sd(as.numeric(intensities_mean_sel[j, rows_test]))
  		comp$test_int_sd[j] <- sd(as.numeric(intensities_mean_sel[j, rows_ctrl]))	  
		#need standard errors of both to calculate the fold change error
		x <- log2(as.numeric(intensities_mean_sel[j, rows_ctrl]))
		x_se <- sqrt(var(x)/length(x))
		y <- log2(as.numeric(intensities_mean_sel[j, rows_test]))
		y_se <- sqrt(var(y)/length(y))
		comp$se_log2[j] <- sqrt(x_se^2+y_se^2)
		comp$log2[j] <- log2(mean(as.numeric(intensities_mean_sel[j, rows_test])) / mean(as.numeric(intensities_mean_sel[j, rows_ctrl])))
  		fit <- unlist(t.test(intensities_mean_sel[j, rows_test], 
  		intensities_mean_sel[j, rows_ctrl]))
  		comp$pval[j] <- as.numeric(fit[3])
	}
	comp$pvalBH <- p.adjust(comp$pval, method="BH")
	ttest_res_list[[i]] <- comp
}
names(ttest_res_list) <- c(seasons, "both")
lapply(ttest_res_list, function(x) {length(which(x$pvalBH <= 0.01))})
ttest_sig_ions_list <- lapply(ttest_res_list, function(x) {which(x$pvalBH <= 0.01)})
#525 fall and 466 in spring, pooling both 645 

length(intersect(ttest_sig_ions_list$fall, ttest_sig_ions_list$spring)) #372 at 0.01
sig_ions_seas_ind <- intersect(ttest_sig_ions_list$fall, ttest_sig_ions_list$spring)
#ions$'Top annotation name'[sig_ions_seas_ind]

#check directionality of change, should all be the same for significant ones 
#plot(ttest_res_list$fall$log2, ttest_res_list$spring$log2)
#points(ttest_res_list$fall$log2[sig_ions_seas_ind], ttest_res_list$spring$log2[sig_ions_seas_ind], pch=1, col="red")
#cor(ttest_res_list$fall$log2, ttest_res_list$spring$log2) #87%
#cor(ttest_res_list$fall$log2[sig_ions_seas_ind], ttest_res_list$spring$log2[sig_ions_seas_ind]) #0.96 in log2 FC
length(sig_ions_seas_ind) #372

head(ttest_res_list[[1]])

#checking the % of standard error for the sig_ions_seas_ind ions
#hist(ttest_res_list[[3]]$se_log2[sig_ions_seas_ind] / abs(ttest_res_list[[3]]$log2[sig_ions_seas_ind]))

#----------------------------------------
#add statistics from the ttest to the ions file
names(ttest_res_list)

head(ttest_res_list[[1]])

one_temp <- ttest_res_list[[1]][,c(2:8)]
two_temp <- ttest_res_list[[2]][,c(2:8)]
three_temp <- ttest_res_list[[3]][,c(2:8)]

temp_name <- c("log2", "se_log2", "ctr_int", "test_int", "ctr_int_sd", "test_int_sd", "pvalBH")

names(one_temp) <- paste(temp_name, "fall", sep="_")
names(two_temp) <- paste(temp_name, "spring", sep="_")
names(three_temp) <- paste(temp_name, "pooled", sep="_")

ions <- cbind(ions, one_temp, two_temp, three_temp)
ions$robust_changes <- "-"
ions$robust_changes[sig_ions_seas_ind] <- "x"
head(ions)

#---------------------------------------------------------------------------------------
#t-tests on intensities of fall MF and hive 

ind <- grep("MF|hive", injections_mean$full_sample_group)
ind <- intersect(ind, col_fall)
intensities_mean_sel <- intensities_mean[,ind] 
injections_mean_sel <- injections_mean[ind,]
names(intensities_mean_sel) <- injections_mean$full_sample_group[ind]
table(names(intensities_mean_sel))

#Gut hive fall   Gut MF fall 
#            5            13 
#----------------------------------------

rows_test <- which(injections_mean_sel$group == "hive")
rows_ctrl <- which(injections_mean_sel$group == "MF")
	
ttest_res_hive_mf <- list()
pval <- seq(1, 1, length.out=nrow(intensities_mean_sel))
log2 <- seq(0, 0, length.out=nrow(intensities_mean_sel))

ttest_res_hive_mf <- data.frame(pval, log2)
ttest_res_hive_mf$se_log2 <- 0
ttest_res_hive_mf$ctr_int <- 0
ttest_res_hive_mf$test_int <- 0
ttest_res_hive_mf$ctr_int_sd <- 0
ttest_res_hive_mf$test_int_sd <- 0
for (j in 1:nrow(intensities_mean_sel)) {
	ttest_res_hive_mf$ctr_int[j] <- round(mean(as.numeric(intensities_mean_sel[j, 
 	rows_test])), digits=0)
  	ttest_res_hive_mf$test_int[j] <-round(mean(as.numeric(intensities_mean_sel[j, 
  	rows_ctrl])), digits=0)  		  	
	ttest_res_hive_mf$ctr_int_sd[j] <- sd(as.numeric(intensities_mean_sel[j, 
	rows_test]))
  	ttest_res_hive_mf$test_int_sd[j] <- sd(as.numeric(intensities_mean_sel[j, 
  	rows_ctrl]))	  
	#need standard errors of both to calculate the fold change error
	x <- log2(as.numeric(intensities_mean_sel[j, rows_ctrl]))
	x_se <- sqrt(var(x)/length(x))
	y <- log2(as.numeric(intensities_mean_sel[j, rows_test]))
	y_se <- sqrt(var(y)/length(y))
	ttest_res_hive_mf$se_log2[j] <- sqrt(x_se^2+y_se^2)
  	ttest_res_hive_mf $log2[j] <- log2(mean(as.numeric(intensities_mean_sel[j, 
  	rows_test])) / mean(as.numeric(intensities_mean_sel[j, rows_ctrl])))
  	fit <- unlist(t.test(intensities_mean_sel[j, rows_test], 
  	intensities_mean_sel[j, rows_ctrl]))
  	ttest_res_hive_mf$pval[j] <- as.numeric(fit[3])
}
ttest_res_hive_mf$pvalBH <- p.adjust(ttest_res_hive_mf$pval, method="BH")
head(ttest_res_hive_mf)

ion_names <- c()
ion_names_temp <- strsplit(ions$'Top annotation name', ";", fixed=TRUE)
ion_names <- do.call(rbind, ion_names_temp)
ion_names_first <- ion_names[,1]

vignettes <- c(67,285,648,654,736,646,995,835)
cbind(ion_names_first[vignettes], ttest_res_hive_mf[vignettes,])

ttest_res_hive_mf_col <- ttest_res_hive_mf[,c(2:8)]
names(ttest_res_hive_mf_col) <- paste(temp_name, "hive_vs_MF", sep="_")
ions_temp <- cbind(ions, ttest_res_hive_mf_col)
head(ions_temp)

#write.table(ions_temp, "hive MF comparison 20170305.tsv", row.names=F, quote=FALSE, sep='\t')
#write.table(ions_temp, "all ions with sd and se 20170403.tsv", row.names=F, quote=FALSE, sep='\t')

#---------------------------------------------------------------------------------------
#enrichment analyses on all and both groups 

#---------------------------------------
#splitting up products and substrates

length(ttest_res_list)
head(ttest_res_list$both) #can be used to find the products and substrates, and overall log2 FC

sig_ions_substrates_ind <- sig_ions_seas_ind[ttest_res_list$both$log2[sig_ions_seas_ind] < 0]
sig_ions_products_ind <- sig_ions_seas_ind[ttest_res_list$both$log2[sig_ions_seas_ind] > 0]
length(sig_ions_substrates_ind) #240
length(sig_ions_products_ind) #132
length(sig_ions_substrates_ind) + length(sig_ions_products_ind) #372

sig_ionIdx_list <- list(sig_ions_seas_ind, sig_ions_substrates_ind, sig_ions_products_ind)
names(sig_ionIdx_list) <- c("sig_ionIdx_all", "sig_ionIdx_subs", "sig_ionIdx_prod")

#---------------------------------------

source("initiate_enrichment_analyses_v2.R") #reads in mo file, makes contingency table, and adds detailed_compound_class to annotation 
source("compute_fishers_enrichment_v2.R")
enriched_list <- compute_fishers_enrichment(sig_ionIdx_list)
names(enriched_list)
length(enriched_list) #first is the 0.01, second is the 0.05 cutoff, third is all

enriched_list[[1]] #at 0.01 #only groups with more than 1 ion are included
enriched_list[[2]] #at 0.05 #only groups with more than 1 ion are included
enriched_list[[3]] #all #only groups with more than 1 ion are included

#more than five per group
enriched_list[[2]][[1]][enriched_list[[2]][[1]]$n_in_ion_list > 5,]

#---------------------------------------
#matching the ions with the unique annotations file, for sign ions
selrows <- list()
for (i in 1:length(sig_ions_seas_ind)) {
  rows <- which(annotation$ionIdx == sig_ions_seas_ind[i])
  selrows[[i]] <- rows[1]
}
gut_sig_ions_ann <- unlist(selrows)

ions_temp <- ions[sig_ions_seas_ind,]
ions_temp$compound_class <- annotation$detailed_compound_class[gut_sig_ions_ann]
ions_temp$direction <- "-"
ions_temp$direction[ions_temp$ionIdx %in% sig_ions_substrates_ind] <- "substrates"
ions_temp$direction[ions_temp$ionIdx %in% sig_ions_products_ind] <- "products"
#write.table(ions_temp, "sign ions 20170228.tsv", row.names=F, quote=FALSE, sep='\t')

#for all ions
selrows <- list()
for (i in 1:length(ions$ionIdx)) {
  rows <- which(annotation$ionIdx == ions$ionIdx[i])
  selrows[[i]] <- rows[1]
}
gut_ions_ann <- unlist(selrows)

ions$compound_class <- annotation$detailed_compound_class[gut_ions_ann]
#write.table(ions, "ions 20170303 with compound class.tsv", row.names=F, quote=FALSE, sep='\t')
#------------------------------------------------------------------------------------------------------
#plotting everything in one figure, oplsda plot

#see oplsda S plot code

#------------------------------------------------------------------------------------------------------  
#3. pollen vs water & plotting, 
#take into account pollen ions from both experiments

a <- grep("beads", injections_mean$full_sample_group)
b <- grep("Pollen", injections_mean$full_sample_group)
ind <- c(a,b)

col_fall <- which(injections_mean$season == "fall")
col_spring <- which(injections_mean$season == "spring")
season_cols <- list(col_fall, col_spring)
names(season_cols) <- c("fall", "spring")
pollen_comp_list <- list()
pollen_ion_ids_list <- list()

for (i in 1:3) {
	if (i == 3) { col_season <- c(col_fall, col_spring) } 
	else { 	col_season <- season_cols[[i]] }
	pollen_ctr_rows <- intersect(a, col_season)
	pollen_test_rows <- intersect(b, col_season) 
	pval <- seq(1, 1, length.out=nrow(intensities_mean))
	log2 <- seq(0, 0, length.out=nrow(intensities_mean))
	pollen_comp <- data.frame(pval, log2)
	pollen_comp$ctr_int <- "-"
	pollen_comp$test_int <- "-"
	for (j in 1:nrow(intensities_mean)) {
  		pollen_comp$ctr_int[j] <- 
  		round(mean(as.numeric(intensities_mean[j,pollen_ctr_rows])), digits=0)
  		pollen_comp$test_int[j] <- 
  		round(mean(as.numeric(intensities_mean[j,pollen_test_rows])), digits=0)
  		pollen_comp$log2[j] <- 	
  		log2(mean(as.numeric(intensities_mean[j,pollen_test_rows])) /
  		mean(as.numeric(intensities_mean[j,pollen_ctr_rows])))
  		fit <- unlist(t.test(intensities_mean[j,pollen_ctr_rows], 
  		intensities_mean[j,pollen_test_rows]))
  		pollen_comp$pval[j] <- as.numeric(fit[3])
	}
	pollen_comp$pvalBH <- p.adjust(pollen_comp$pval, method="BH")
	pollen_ion_ids <- which(pollen_comp$log2 >= 2 & pollen_comp$pvalBH <= 0.001 &
          as.numeric(pollen_comp$test_int) >= 5000 & as.numeric(ions$'Top annotation score' == 100))
	pollen_comp_list[[i]] <- pollen_comp
	pollen_ion_ids_list[[i]] <- pollen_ion_ids
}                          
names(pollen_ion_ids_list) <- c(names(season_cols), "combined")
names(pollen_comp_list) <- c(names(season_cols), "combined")
lapply(pollen_ion_ids_list, FUN=length) #514 fall, 476 spring, 508 combined
#pollen_ion_ids_season <- intersect(pollen_ion_ids_list[[1]], pollen_ion_ids_list[[2]])
pollen_ion_ids_comb <- pollen_ion_ids_list$combined
length(pollen_ion_ids_comb) #508, more filtering below to get to pollen_ion_ids_season

#------------------------------------------------
#aggregate log2 and pvalBH across pollen_comp_list
pollen_log2 <- pollen_comp_list$combined$log2
pollen_test_int <- as.numeric(pollen_comp_list$combined$test_int)
pollen_pvalBH <- pollen_comp_list$combined$pvalBH

#------------------------------------------------
#select only with reasonable mean intensity and top annotation score to be more certain there ions are really present in pollen

to_include <- which(pollen_test_int >= 10000 & as.numeric(ions$'Top annotation score' == 100))
pollen_ion_ids_season <- intersect(pollen_ion_ids_comb, to_include)
length(pollen_ion_ids_season) #402 pollen ions


#make pollen_ion_ids_ann with the rows from the annotation
selrows <- list()
for (i in 1:length(pollen_ion_ids_season)) {
  rows <- which(annotation$ionIdx == pollen_ion_ids_season[i])
  selrows[[i]] <- rows[1]
}
pollen_ion_ids_ann <- unlist(selrows)

#------------------------------------------------
#enrichment on pollen ions

source("initiate_enrichment_analyses_v2.R") 
source("compute_fishers_enrichment_v2.R")
enriched_list <- compute_fishers_enrichment(list(pollen_ion_ids_season))

names(enriched_list)
length(enriched_list) #first is the 0.01, second is the 0.05 cutoff, third is all

enriched_list[[1]] #at 0.01 #only groups with more than 1 ion are included
enriched_list[[2]] #at 0.05 #only groups with more than 1 ion are included
enriched_list[[3]] #all #only groups with more than 1 ion are included

#more than five per group
enriched_list[[2]][[1]][enriched_list[[2]][[1]]$n_in_ion_list > 5,]

#-----------------------------------
pdf("pollen ions volcano seasonal v2.pdf", width=5, height=4)
par(mar=c(3.5,3.5,1,1))
plot(pollen_log2[to_include], -log10(pollen_pvalBH[to_include]), 
        ylab="", xlab="", las=1, pch=16, cex=0.8, col=rgb(0.6, 0.6, 0.6, 0.5), xlim=c(-9,12))
abline(v=2, col=rgb(0.6, 0.6, 0.6, 0.5), lwd=2) #for log2 FC
abline(v=-2, col=rgb(0.6, 0.6, 0.6, 0.5), lwd=2) #for log2 FC
abline(h=3, col=rgb(0.6, 0.6, 0.6, 0.5), lwd=2) #for -log10 p value
points(pollen_log2[pollen_ion_ids_season], -log10(pollen_pvalBH[pollen_ion_ids_season]), pch=16, col="black", cex=0.8)

pollen_groups <- c("Amino Acids and Derivatives", "Carboxylic Acids and Derivatives", "Flavonoids", "Monosaccharides", "Pyrimidine Nucleosides and Analogues", "Purine Nucleosides and Analogues")
colors_unique <- c(brewer.pal(5, "Dark2"), brewer.pal(5, "Dark2")[5]) #last color twice for nucleosides
for (i in 1:length(pollen_groups)) {
  rows <- pollen_ion_ids_season[grep(pollen_groups[i], annotation$detailed_compound_class[pollen_ion_ids_ann])]  
  points(pollen_log2[rows], -log10(pollen_pvalBH[rows]), pch=16, col= colors_unique[i], cex=0.8)
}
pollen_group_names <- c("Amino Acids & Derivatives", "Carboxylic Acids & Derivatives", "Flavonoids", "Monosaccharides", "Nucleosides & Analogs")
legend(-9.8, 19, legend= pollen_group_names[1:5], pch=16, col= colors_unique[1:5], cex=0.8, bty="n")
mtext("log2(fold change)", side=1, line=2.5)
mtext("-log10(p-value)", side=2, line=2.5)
dev.off()

#-----------------------------------------------------------------------------------------------------
#write.table(annotation[which(annotation$ionIdx %in% pollen_ion_ids_season),], "ions higher in pollen_full annotation Dec.tsv", row.names=F, quote=FALSE, sep='\t')
#write.table(ions[pollen_ion_ids_season,], "ions higher in pollen Dec.tsv", row.names=F, quote=FALSE, sep='\t')
#----------------------------------------
#numeric data for the volcano plot

plot_or_not <- rep("no", times=length(pollen_log2))
plot_or_not[to_include] <- "yes"
pollen_ion <- rep("no", times=length(pollen_log2))
pollen_ion[pollen_ion_ids_season] <- "yes"

pollen_volcano_data <- as.data.frame(cbind(pollen_log2, pollen_pvalBH, plot_or_not, pollen_ion))
#write.csv(pollen_volcano_data, "pollen_volcano_data.csv", row.names=F)

#----------------------------------------
#monocolonizations analyses, which changes are explained by which bacterium?

#selected dataframe with only relevant groups and fall data, fall CL and MF only

selected_ds_names <- unique(injections_mean$group)[c(1,3:11)]
selected_ds_ids <- which(injections_mean$group %in% selected_ds_names)
sub_rows <- intersect(grep("MF|CL", injections_mean$group[selected_ds_ids]), which(injections_mean$season[selected_ds_ids] == "spring"))
injections_mean_sel <- injections_mean[selected_ds_ids[-sub_rows],]
intensities_mean_sel <- intensities_mean[, selected_ds_ids[-sub_rows]]
table(injections_mean_sel$group)

#number of bees used for monocol analysis
#  A1    B   CL   F4   F5   Fp    G hive   MF    S 
#   5    5   12    5    5    9    5    5   13    9 

#------------------------------------------------------------------------------------------------------
#per ion compute Z-score

intensities_gut_zscore <- intensities_mean_sel #to get proper dimensions
for (i in 1:nrow(intensities_gut_zscore)) {
  	intensities_gut_zscore[i,] <- (intensities_mean_sel[i,] - mean(as.numeric(intensities_mean_sel[i,]))) / sd(as.numeric(intensities_mean_sel[i,]))
}
dim(intensities_gut_zscore)

#hist(as.numeric(unlist(intensities_gut_zscore)), breaks=100) #normal distribution
#--------------------------------------------
#write these to file for plos biology

dim(intensities_gut_zscore)
dim(injections_mean_sel)

#write.csv(intensities_gut_zscore, "intensities_gut_zscores.csv", row.names=F)
#write.csv(injections_mean_sel, "injections_mean_sel.csv", row.names=F)

#------------------------------------------------------------------------------------------------------  
#anova

anova_res_list <- list()
anova_ind_list <- list()
intensities_sub_zcore <- intensities_gut_zscore
injections_sub <- injections_mean_sel
res_list <- list()
ind_list <- list()
for (j in 1:nrow(intensities_sub_zcore)) {
  	fit <- aov(as.numeric(intensities_sub_zcore[j,]) ~ injections_sub$group)
  	temp <- TukeyHSD(fit, conf.level=0.99, ordered=F) 
  	#important to make false, every time same order
  	res_list[[j]] <- data.frame(temp$`injections_sub$group`) #extracts the data  
  	ind_list[[j]] <-  which(rownames(res_list[[j]]) %in% c("MF-CL", "CL-MF"))
}
anova_res_list <- res_list
anova_ind_list <- unique(unlist(ind_list))

length(anova_res_list)
anova_res_list[[1]]

#--------------------------------------------
#ordering the anova results

#groups <- sort(unique(injections_sub$group))
groups <- unique(injections_sub$group)[order(tolower(unique(injections_sub$group)))]
groups_PB_names <- c("Ba", "Bi", "CL", "F4", "F5", "Fp", "Ga", "Hive", "MD", "Sa" )

groups_list_ion_aov_mf <- list()
for (i in 1:nrow(intensities_sub_zcore)) {
	comparisons_mf <- c(paste("MF-", groups, sep=""), paste(groups, "-MF", sep=""))
	temp_ind <- which(rownames(res_list[[i]]) %in% comparisons_mf)
	temp_names <- rownames(res_list[[i]])[temp_ind] #is the order
	#put in order of groups
	#correct order of S, reversed the three columns, not the p-value
	row_s <- grep("S", temp_names)
	res_list[[i]][temp_ind[row_s],1:3] <- -res_list[[i]][temp_ind[row_s],1:3]
	res_df_temp <- res_list[[i]][temp_ind,]
	res_df_temp$names <- gsub("-", "", gsub("MF", "", temp_names))
 	MF_vec <- c(0,0,0,0,"MF")
	groups_list_ion_aov_mf[[i]] <- rbind(res_df_temp[1:8,], MF_vec, res_df_temp[9,])
	groups_list_ion_aov_mf[[i]]$names_PB <- groups_PB_names
}

#------------------------------------------------------------------------------------------------------
#make groups_list_ion_zcore that has all the Z-scores in a list format

groups_list_ion_zcore <- vector("list", length=nrow(intensities_sub_zcore))
for (i in 1:nrow(intensities_sub_zcore)) {
	for (j in 1:length(groups)) {
		temp_col <- which(injections_mean_sel$group == groups[j])
		groups_list_ion_zcore[[i]][[j]] <- as.numeric(intensities_sub_zcore[i, 
			temp_col])
	}
}		
#------------------------------------------------------------------------------------------------------
#list with the the phyla and colors

require(RColorBrewer)
colors_unique <- brewer.pal(7, "Dark2") #Dark2 is colorblind friendly
black <- brewer.pal(11, "RdGy")[11]

phyla <- c("Alpha", "Bifido", "CL", "Firm" , "Firm", "Gamma", "Gamma", "hive", "MF", "Beta-proteo") 
length(unique(phyla))
length(phyla)

unique(phyla)
#should have length of groups
group_colors <- c(colors_unique[6], 
	colors_unique[4],
	black,
	colors_unique[5],
	colors_unique[5],
	colors_unique[7],
	colors_unique[7],
	colors_unique[3], 
	black,
	colors_unique[2])

unique(phyla)
unique(group_colors)
pdf("phyla colors legend.pdf", width=2.5, height=3.5)
par(mar=c(1,1,1,1))		
plot(1:10, type="n", axes=F, xlab="", ylab="")
legend(1, 10, legend=phyla , pch="", group_colors, cex=1.2, bty="n")
dev.off()

#------------------------------------------------------------------------------------------------------
#first loop over the results to identify explained ions and add the groups
#in the second loop produce the plots that meet the criteria

ion_names <- c()
#ion_names_temp <- strsplit(ions$'Top annotation name', ";", fixed=TRUE)
#ion_names <- do.call(rbind, ion_names_temp)
#ion_names_first <- ion_names[,1]
ion_names_first <- ions$curated_annotation_sign
ion_nr_ann <- unlist(lapply(strsplit(ions$'Top annotation ids', ";", fixed=TRUE), FUN=length))
ion_names_first[ion_nr_ann > 1] <- paste(ion_names_first[ion_nr_ann > 1], "*", sep="")

#remove the * for afzelin, quercitrin, and rutin c(821, 835, 995)
ion_names_first_no_star <- gsub("\\*", "", ion_names_first)

#------------------------------------------
#plotting and explained analyses

#Make a seperate folder, or add plots to your current working directory
setwd("/Users/admin/Downloads/Kesnerova_et_al_PB_2017_R_analyses")

ions_id_sel <- sig_ions_seas_ind
plot_data_explained <- vector("list", length=length(sig_ions_seas_ind))

ion_id_explained <- c()
ion_id_all_group <- c()
ion_id_explained_group <- c()
ion_id_explained_monocols <- list()
ion_id_explained_mf_cl_dif <- c()

pos_groups_mf <- which(groups == "MF") #9
pos_groups_cl <- which(groups == "CL") #3
ordering_mf <- c(pos_groups_mf, setdiff(1:length(groups), c(pos_groups_cl, pos_groups_mf)), pos_groups_cl)

to_plot_or_not_to_plot <- FALSE #use false if you do not want to plot but just run the explained analysis
use_new_names <- FALSE #use false if you want to run the explained part of the rest of the code since this uses the old short names (e.g "B" instead of "Bi")

k=0
for (i in 1:length(ions_id_sel)) {
  ion_id <- ions_id_sel[i]  
  data_stats_mf <- groups_list_ion_aov_mf[[ion_id]]
  data_z_score <- groups_list_ion_zcore[[ion_id]]

  means_ord <- unlist(lapply(data_z_score, FUN=mean)[ordering_mf])
  data_z_score_ord_temp <- data_z_score[ordering_mf]
  data_stats_ord_temp_mf <- data_stats_mf[ordering_mf,]
  group_colors_ord_temp <- group_colors[ordering_mf] 

  if (use_new_names == TRUE) {
	  groups_ord_temp <- data_stats_ord_temp_mf$names_PB
	} else { groups_ord_temp <- data_stats_ord_temp_mf$names }

  #group by patterns, MF is bigger substrate, MF is larger product
  if (means_ord[1] > means_ord[length(means_ord)]) {
    metabolite_group <- "microbial substrate"
  } else { metabolite_group <- "microbial product"}
  ion_id_all_group <- append(ion_id_all_group, metabolite_group)
  
  #order middle rows by mean Z-score, +1 because one row removed
  if (means_ord[1] > means_ord[length(means_ord)]) {
    order_by_mean_z <- order(means_ord[2:(length(means_ord)-1)], decreasing=T)+1 
     } else { order_by_mean_z <- order(means_ord[2:(length(means_ord)-1)], 
     	decreasing=F)+1
  }
  ngroups <- nrow(groups_list_ion_aov_mf[[ion_id]])
  data_z_score_ord <- data_z_score_ord_temp[c(1, order_by_mean_z, ngroups)]
  groups_ord <- groups_ord_temp[c(1, order_by_mean_z, ngroups)]
  data_stats_ord_mf <- data_stats_ord_temp_mf[c(1, order_by_mean_z, ngroups),]
  group_colors_ord <- group_colors_ord_temp[c(1, order_by_mean_z, ngroups)]
    
  xmax <- ceiling(max(unlist(groups_list_ion_zcore[[ion_id]])))
  xmin <- floor(min(unlist(groups_list_ion_zcore[[ion_id]])))
  
#with shading of color when not significant 
  means_rev <- rev(as.numeric(lapply(data_z_score_ord, FUN=mean)))
  means_mf_diff <- means_rev - means_rev[1]
  means_cl_diff <- means_rev - means_rev[ngroups]
  dif_cl_mf <- abs(means_rev[ngroups] - means_rev[1])
  significant_ones <- which(as.numeric(data_stats_ord_mf$p.adj) < 0.05)
  
  means <- as.numeric(lapply(data_z_score_ord, FUN=mean))
  to_remove <- c()
  if (metabolite_group == "microbial product") {
  to_remove <- which(means < means[1]) #MF is first one
  significant_ones  <- setdiff(significant_ones, to_remove) 
  } else {  to_remove <- which(means > means[1])
  significant_ones  <- setdiff(significant_ones, to_remove) }
  
  group_colors_ord[setdiff(1:ngroups, significant_ones)] <- rgb(0.6, 0.6, 0.6, 0.5)
  group_colors_ord[c(1,10)] <- "black" 

  if (length(significant_ones) >= 1) { #plot all
    k <- k+1
    ion_id_explained <- append(ion_id_explained, ion_id)
    ion_id_explained_group <- append(ion_id_explained_group, metabolite_group)
	sig_groups_temp <- groups_ord[group_colors_ord != "#99999980"]
    ion_id_explained_monocols[[k]] <- sig_groups_temp[2:(length(sig_groups_temp)-1)]
	ion_id_explained_mf_cl_dif <- c(ion_id_explained_mf_cl_dif, (means[ngroups] - means[1]))

    plotname_short <- paste(ion_id, metabolite_group,ion_names_first_no_star[ion_id], sep="  ")
    plotname <- paste(ion_names_first[ion_id], "\n" , "#", ion_id, substr(ions$ionMz[ion_id], 1, 7), metabolite_group, sep="  ")
    filename <- paste(plotname_short, ".pdf", sep="")
    filename <- gsub(">", "", filename)
    
    if (to_plot_or_not_to_plot==TRUE) {
    pdf(filename, width=5, height=3.5)
    par(mar=c(3.5,4.5,3,1))
    plot(seq(0, xmax, by=xmax/(length(1:ngroups)-1)), 1:ngroups, xlim=c(xmin, xmax), type="n", ylab="", xlab="", yaxt="n", xaxt="n") #initiate empty plot
    axis(2, length(data_z_score_ord):1,  groups_ord, las=1)
    axis(1)
    mtext("ion Z-score", side=1, line=2.5)
    title(plotname, line=0.7)
    data_z_score_ord <- data_z_score_ord[ngroups:1]
    
    for (j in 1:length(data_z_score_ord)) {
      points(data_z_score_ord[[j]], rep(j, length(data_z_score_ord[[j]])), pch=16, col=rev(group_colors_ord)[j])
      segments(mean(data_z_score_ord[[j]]), j-0.3, mean(data_z_score_ord[[j]]), j+0.3, lwd=2, col=rev(group_colors_ord)[j])
    
    }
    dev.off()
  	}
}
plot_data_explained[[i]] <- data_z_score_ord
names(plot_data_explained[[i]]) <- groups_ord  
}  
names(plot_data_explained) <- sig_ions_seas_ind
#save(file="plot_data_explained", plot_data_explained)

#there are some cases where MF CL diffence was not significant, I remove these here
ion_id_explained_monocols_first <- lapply(ion_id_explained_monocols, function(x) {gsub("MF", NA, x)})
for (i in 1:length(ion_id_explained_monocols_first)) {
	if (NA %in% ion_id_explained_monocols_first[[i]]) {
		ion_id_explained_monocols_first[[i]] <- na.omit(ion_id_explained_monocols_first[[i]])[1]
	}
}
ion_id_explained_monocols <- ion_id_explained_monocols_first

#optionally set working directory back
setwd("/Users/admin/Downloads/Kesnerova_et_al_PB_2017_R_analyses")

#-----------------------------------------------------------------------------------------------------
#sum up the explained changes

length(ion_id_explained) #all are now plotted
length(ion_id_explained_group)
length(ion_id_explained_monocols)
length(ion_id_explained_mf_cl_dif)

ions_expl <- as.data.frame(cbind(ion_id_explained, ion_id_explained_group))
names(ions_expl) <- c("idx", "group")
table(ions_expl$group) #the MF : CL comparison classifies 132 products, 240 substrates, no hive criterion
#MF : CL comparisons that are the same in hive
hive_ions <- which(lapply(ion_id_explained_monocols, FUN=function(x) {grep("hive", x)}) >= 1)
table(ions_expl$group[hive_ions]) #91 products, 196 substrates

#summing up the similarity with MF and CL
table(unlist(ion_id_explained_monocols)) #note, because of the way the loop is set up the CL numbers are not accurate
#  A1    B   CL   F4   F5   Fp    G hive    S 
#  20   96   28  126  170   65   61  287   45

ions_df_expl <- ions[as.character(ions_expl$idx),]
ions_df_expl$sig_monocols <- ion_id_explained_monocols
ions_df_expl$sig_monocols <- sapply(ions_df_expl$sig_monocols, FUN=paste, collapse = ",")
ions_df_expl$group <- ion_id_explained_group
ions_df_expl$MF_CL_deltaZscore <- ion_id_explained_mf_cl_dif

table(unlist(strsplit(ions_df_expl$sig_monocols[ions_df_expl$group == "microbial substrate"], ",")))
table(unlist(strsplit(ions_df_expl$sig_monocols[ions_df_expl$group == "microbial product"], ",")))

#calculate how many unique ions are explained by all except for hive, 306
singles_expl <- c()
singles <- c("A1", "B", "F4", "F5", "Fp", "G", "S")
for (i in 1:length(ions_df_expl$sig_monocols)) {
  singles_expl <- append(singles_expl, TRUE %in% (singles %in% strsplit(ions_df_expl$sig_monocols[i], ",")[[1]]))
}
explained_all_no_hive <- table(singles_expl)[[2]]

#products and substrates are always more like CL so get a 1 if in there
#meaning that CL gets score of length(ions_id_sel)
df <- as.data.frame(table(unlist(strsplit(ions_df_expl$sig_monocols, ","))), stringsAsFactors=F)
df <- rbind(c("MF", 0), df)
df[df[,1]== "CL",2] <- length(ions_id_sel) #CL in here is a remnant of the ones with only CL : MF difference but not explained from above
df <- rbind(c("unique monocol", explained_all_no_hive), df)
names(df) <- c("group", "score")
df[order(as.numeric(df$score)),]

groups
group_colors_sort <- group_colors
#ordered df
dff <- df[order(df$group),] 
dffgroup <- dff[c(1:10),] #temp remove unique monocol, manual!
dffgroup$colors <- group_colors_sort
dffgroup[(nrow(dffgroup)+1),] <- c(dff[11,], "darkgrey") #manual!

# and the same with distinguish substrates and products
ions_expl_substrate <- ions_df_expl[ions_df_expl$group == "microbial substrate",]
ions_expl_product <- ions_df_expl[ions_df_expl$group == "microbial product",]
#-----------------------------------------------------------------------------------------------------
#writing tsv file
#write.table(ions_df_expl, "seasonal ions data.tsv", row.names=F, quote=FALSE, sep='\t')
#-----------------------------------------------------------------------------------------------------
#check if there are any patterns in the not explained ones with enriched analysis

ion_not_explained <- ion_id_explained[singles_expl == FALSE]
#ions$'Top annotation name'[ion_not_explained]

#enrichment
source("initiate_enrichment_analyses_v2.R") #reads in mo file, makes contingency table, and adds detailed_compound_class to annotation 
source("compute_fishers_enrichment_v2.R")

enriched_list_all <- compute_fishers_enrichment(list(ion_not_explained))
length(enriched_list_all) #first is the 0.01, second is the 0.05 cutoff, third all
enriched_list_all[[2]] #only groups with more than 1 ion are included, at 0.05
enriched_list_all[[3]] #only groups with more than 1 ion are included, all

#-----------------------------------------------------------------------------------------------------
#calculate how many unique ions are explained by all but hive. 
#201 substrates
singles_expl_substrate <- c()
singles <- c("A1", "B", "F4", "F5", "Fp", "G", "S")
for (i in 1:length(ions_expl_substrate$sig_monocols)) {
	singles_expl_substrate <- append(singles_expl_substrate, TRUE %in% (singles %in% strsplit(ions_expl_substrate$sig_monocols[i], ",")[[1]]))
}
table(singles_expl_substrate)
explained_substrate_no_hive <- table(singles_expl_substrate)[[2]]
#-----------------------------------------
#98 products 
singles_expl_product <- c()
for (i in 1:length(ions_expl_product$sig_monocols)) {
	singles_expl_product <- append(singles_expl_product, TRUE %in% (singles %in% strsplit(ions_expl_product$sig_monocols[i], ",")[[1]]))
}
table(singles_expl_product)
explained_product_no_hive <- table(singles_expl_product)[[2]]
#-----------------------------------------

#products and substrates are always more like CL so get a 1 if in there
#meaning that CL gets score of length(ions_id_sel)
df_substrate <- as.data.frame(table(unlist(strsplit(ions_expl_substrate$sig_monocols, ","))), stringsAsFactors=F)
df_product <- as.data.frame(table(unlist(strsplit(ions_expl_product$sig_monocols, ","))), stringsAsFactors=F)
df_substrate <- rbind(c("MF", 0), df_substrate)
df_product <- rbind(c("MF", 0), df_product)
df_substrate[df_substrate[,1]== "CL",2] <- nrow(ions_expl_substrate)
df_product[df_product[,1]== "CL",2] <- nrow(ions_expl_product)
df_substrate <- rbind(c("unique monocol", explained_substrate_no_hive), df_substrate)
df_product <- rbind(c("unique monocol", explained_product_no_hive), df_product)
names(df_substrate) <- c("group", "score")
df_substrate[order(as.numeric(df_substrate$score)),]
names(df_product) <- c("group", "score")
df_product[order(as.numeric(df_product$score)),]


#color by phyla as in the plots above
#sort for final version #colors from above, are in order of groups

group_colors_sort <- group_colors
#ordered df
dff_product <- df_product[order(df_product$group),] 
dffgroup_prod <- dff_product[c(1:(nrow(dff_product)-1)),] #temp remove unique monocol
dffgroup_prod$colors <- group_colors_sort
dffgroup_prod[(nrow(dffgroup_prod)+1),] <- c(dff_product[11,], "darkgrey")

df_substrate <- df_substrate[order(df_substrate$group),] 
dffgroup_subs <- df_substrate[c(1:(nrow(df_substrate)-1)),] #temp remove unique monocol
dffgroup_subs$colors <- group_colors_sort
dffgroup_subs[(nrow(dffgroup_subs)+1),] <- c(df_substrate[11,], "darkgrey")

names_to_plot <- dffgroup$group
names_to_plot <- gsub("unique monocol", "uniq.", names_to_plot)

#-----------------------------------------------
dffgroup[order(as.numeric(dffgroup$score)),c(1,2)]
#              group score
#2                MF     0
#3                A1    20
#11                S    45
#9                 G    61
#8                Fp    65
#4                 B    96
#6                F4   126
#7                F5   170
#10             hive   287
#11.1 unique monocol   299
#5                CL   372


dffgroup_subs[order(as.numeric(dffgroup_subs$score)),c(1,2)]
#              group score
#2                MF     0
#3                A1    15
#11                S    31
#9                 G    44
#4                 B    55
#8                Fp    58
#6                F4   107
#7                F5   126
#10             hive   196
#11.1 unique monocol   201
#5                CL   240


dffgroup_prod[order(as.numeric(dffgroup_prod$score)),c(1,2)]
#              group score
#2                MF     0
#3                A1     5
#8                Fp     7
#11                S    14
#9                 G    17
#6                F4    19
#4                 B    41
#7                F5    44
#10             hive    91
#11.1 unique monocol    98
#5                CL   132

vec_temp <- c(rev(dffgroup_subs[order(as.numeric(dffgroup_subs$score)),c(2)]), 
rev(dffgroup_prod[order(as.numeric(dffgroup_prod$score)),c(2)]))
df_temp <- as.data.frame(vec_temp)
#write.csv(df_temp, "df_temp_barplot.csv")

#-----------------------------------------------
#perform enrichment analysis on all of the explained by monocolonizations
table(unlist(ion_id_explained_monocols))

#add the MF, CL, and unique monocols
#MF is no ions, CL is significant ions, unique monocols is any one without the hive
ion_id_explained_monocols

#on all
monocols_names <- names(table(unlist(ion_id_explained_monocols)))
monocols_ids_list <- vector("list", length(monocols_names))
for (i in 1:length(monocols_names)) {
	for (j in 1:length(ion_id_explained_monocols)) {
	if (monocols_names[i] %in% ion_id_explained_monocols[[j]]) {
		monocols_ids_list[[i]] <- append(monocols_ids_list[[i]], ion_id_explained[j])
	}
	}
}
names(monocols_ids_list) <- monocols_names
monocols_ids_list$MF <- c("")
monocols_ids_list$CL <- sig_ions_seas_ind
monocols_ids_list$'unique monocols' <- ions_df_expl$ionIdx[singles_expl]

#enrichment on this list 
source("initiate_enrichment_analyses_v2.R") #reads in mo file, makes contingency table, and adds detailed_compound_class to annotation 
source("compute_fishers_enrichment_v2.R")

enriched_list_all <- compute_fishers_enrichment(monocols_ids_list)
length(enriched_list_all) #first is the 0.01, second is the 0.05 cutoff, third all
enriched_list_all[[2]] #only groups with more than 1 ion are included, at 0.05
enriched_list_all[[3]] #only groups with more than 1 ion are included, at 0.05

#-----------------------------------------------
#on substrates

ion_id_explained_monocols_substrate <- strsplit(ions_df_expl$sig_monocols[ions_expl$group == "microbial substrate"], ",")

monocols_names <- names(table(unlist(ion_id_explained_monocols_substrate)))
monocols_ids_list_substrate <- vector("list", length(monocols_names))
for (i in 1:length(monocols_names)) {
	for (j in 1:length(ion_id_explained_monocols_substrate)) {
	if (monocols_names[i] %in% ion_id_explained_monocols_substrate[[j]]) {
		monocols_ids_list_substrate[[i]] <- append(monocols_ids_list_substrate[[i]], ion_id_explained[ions_expl$group == "microbial substrate"][j])
	}
	}
}
names(monocols_ids_list_substrate) <- monocols_names

monocols_ids_list_substrate$MF <- c("")
monocols_ids_list_substrate$CL <- ions_expl_substrate$ionIdx
monocols_ids_list_substrate$'unique monocols' <- ions_expl_substrate$ionIdx[singles_expl_substrate]

#enrichment on this list 
enriched_list_subs <- compute_fishers_enrichment(monocols_ids_list_substrate)
enriched_list_subs[[2]] #only groups with more than 1 ion are included, at 0.05
enriched_list_subs[[3]]
#-----------------------------------------------
#on products
ion_id_explained_monocols_product <- strsplit(ions_df_expl$sig_monocols[ions_expl$group == "microbial product"], ",")

monocols_names <- names(table(unlist(ion_id_explained_monocols_product)))
monocols_ids_list_product <- vector("list", length(monocols_names))
for (i in 1:length(monocols_names)) {
	for (j in 1:length(ion_id_explained_monocols_product)) {
	if (monocols_names[i] %in% ion_id_explained_monocols_product[[j]]) {
		monocols_ids_list_product[[i]] <- append(monocols_ids_list_product[[i]], ion_id_explained[ions_expl$group == "microbial product"][j])
	}
	}
}
names(monocols_ids_list_product) <- monocols_names

monocols_ids_list_product$MF <- c("")
monocols_ids_list_product$CL <- ions_expl_product$ionIdx
monocols_ids_list_product$'unique monocols' <- ions_expl_product$ionIdx[singles_expl_substrate]

#enrichment on this list 
enriched_list_prod <- compute_fishers_enrichment(monocols_ids_list_product)
enriched_list_prod[[2]] #only groups with more than 1 ion are included, at 0.05
enriched_list_prod[[3]] 
#-----------------------------------------------
#incorporate all unique categories in the heatmap to make an overall heatmap for all, substrates, and products

require(gplots)
require(RColorBrewer)

#order the lists here by their name, same order as in dff to make comparible with barplots

combined_enriched <- list(
enriched_list_all[[2]][order(names(enriched_list_all[[2]]))], 
enriched_list_subs[[2]][order(names(enriched_list_subs[[2]]))], 
enriched_list_prod[[2]][order(names(enriched_list_prod[[2]]))])

categories <- unique(unlist(lapply(combined_enriched, function(x) {lapply(x, function(z) {as.character(z$group_name)})})))
species <- names(enriched_list_all[[2]][order(names(enriched_list_all[[2]]))])

#replace with p value

enrich_mat_list <- vector("list",3)
for (j in 1:length(combined_enriched)) { 
	enrich_mat <- matrix(data=1, ncol=length(categories), nrow=length(species))
	#iterate over species, fill if match with category
	enriched_data_list <- combined_enriched[[j]]
	for (i in 1:length(species)) {
		cols <- which(categories %in% enriched_data_list[[i]]$group_name)
		rows <- which(enriched_data_list[[i]]$group_name %in% categories)
		enrich_mat[i, cols] <- unlist(enriched_data_list[[i]]$Fishers_enrichment_pval[rows])
	}
	enrich_mat_list[[j]] <- enrich_mat
}
fill <- 2
enrich_mat_comb <- do.call(rbind, enrich_mat_list)
fill_mat <- matrix(data=1, ncol=length(categories), nrow=fill)
#order by dff

enrich_mat_comb_fill <- rbind(
enrich_mat_comb[1:11,][rev(order(as.numeric(dffgroup$score))),], fill_mat, 
enrich_mat_comb[12:22,][rev(order(as.numeric(dffgroup_subs$score))),], fill_mat, 
enrich_mat_comb[23:33,][rev(order(as.numeric(dffgroup_prod$score))),])

fill_name <- rep("", length=fill)
species_name <- c(species[rev(order(as.numeric(dffgroup$score)))], fill_name, 
species[rev(order(as.numeric(dffgroup_subs$score)))], fill_name, 
species[rev(order(as.numeric(dffgroup_prod$score)))])

species_name <- gsub("unique monocols", "total", species_name)
col_palette <- colorRampPalette(c("black", "white"))(10)
#------------------------------------------------------------------------------------
#start of plot, heatmap and barplot combined
#full, or with only selected categories
#------------------------------------
#selected categories
enrich_mat_comb_fill_backup <- enrich_mat_comb_fill
categories_backup <- categories
categories <- categories[c(1:4,6,7,10,18)]
enrich_mat_comb_fill <- enrich_mat_comb_fill[,c(1:4,6,7,10,18)]
#------------------------------------

any_mat <- enrich_mat_comb_fill
if (ncol(any_mat) > nrow(any_mat)) {
	x_dim <- ncol(any_mat)
} else {x_dim <- nrow(any_mat)}

#to do the reordering using heatmap2
hm <- heatmap.2(enrich_mat_comb_fill, dendrogram="none", trace="none", hline="", Rowv=FALSE, symm=FALSE, colsep=1:ncol(enrich_mat_comb_fill), rowsep=1:nrow(enrich_mat_comb_fill), col=col_palette, breaks=seq(0, 0.1, length.out= 11), symbreaks=FALSE, labCol="", labRow= species_name, srtCol=45, margins=c(0,3), cexCol=0.8, cexRow=0.6, lhei=c(1.5,4), lwid=c(1,6), key=F)
dev.off()

any_mat <- any_mat[,hm$colInd]

#fraction heatmap
frac <- 0.45 
m <- rbind(c(0, frac, 0, 1), c(frac, 1, 0, 1))
dev.off()
split.screen(m)
dev.off()


#this plot has all, substrates, and products. For the manuscript all was removed and names were updated in illustrator

width_corrected <- 3*(frac+0.5)
pdf("heatmap and barplot selected cats v3.pdf", width= width_corrected, height=5.5)
#pdf("heatmap and barplot all cats v3.pdf", width= width_corrected, height=5.5)

split.screen(m)

screen(1)
par(mar=c(1,0,6,0))
plot(1:x_dim, 1:x_dim, pch="", ylim=c(nrow(any_mat),2), xlim=c(0, ncol(any_mat)+2), axes=F, xlab="", ylab="")
colvec <- colorRampPalette(c('black','white'))
#This adds a column of color values
# based on the y values
any_mat[any_mat == 1] <- 0.1
for (j in 1:nrow(any_mat)) {
	colvec_row_temp <- colvec(100)[as.numeric(cut(any_mat[j,],breaks = 100))]
	test <- length(unique(any_mat[j,]))
	if (test == 1) { colvec_row_temp <- "#FFFFFF" }
	points(1:ncol(any_mat), rep(j, ncol(any_mat)), pch=15, col= colvec_row_temp)
}
#for legend 
colvec_legend_temp <- colvec(100)[as.numeric(cut(seq(0,0.1, length=8),breaks = 100))]

#horizontal segments
grid_lwd <- 0.7
#11 groups, 11 segments per block (only on bottom)
y_coord <- seq(0.5, nrow(any_mat)+0.5, length=nrow(any_mat)+1)[c(1:12, 14:25, 27:38)]
#make segments
for (i in 1:length(y_coord)) {
	segments(0.5, y_coord[i], ncol(any_mat)+0.5, y_coord[i], lwd= grid_lwd, col="darkgrey")
}
x_coord <- seq(0.5, ncol(any_mat)+0.5, length=ncol(any_mat)+1)
for (i in 1:length(x_coord)) {
	segments(x_coord[i], 0.5, x_coord[i], nrow(any_mat)+0.5, lwd= grid_lwd, col="darkgrey")
}
names_cats <- categories[hm$colInd]
names_cats <- gsub("and Analogues", "& An.",names_cats)
names_cats <- gsub("and Derivatives", "& Der.",names_cats)

text(1:ncol(any_mat), par("usr")[1] + 0.5, labels=names_cats, cex=0.7, xpd=TRUE, adj=0, srt=45)
text(par("usr")[1] + ncol(any_mat)+1.5, 1:nrow(any_mat), labels=species_name, cex=0.7, xpd=TRUE, adj=0)

#sorting the barplot in the same way
dff_mat_comb <- rbind( dffgroup[rev(order(as.numeric(dffgroup$score))),], 
dffgroup_subs[rev(order(as.numeric(dffgroup_subs$score))),],
dffgroup_prod[rev(order(as.numeric(dffgroup_prod $score))),])

fill_mat <- as.data.frame(cbind(rep("", fill), rep(0, fill), rep("white", fill)))
names(fill_mat) <- names(dff_mat_comb)
dff_mat_comb_fill <- rbind(dff_mat_comb[1:11,], fill_mat, dff_mat_comb[12:22,], fill_mat, dff_mat_comb[23:33,])

dff_mat_comb_fill$score <- as.numeric(dff_mat_comb_fill$score)
dff_mat_comb_fill$x_start <- 0
#400 is max, x_dim is max
dff_mat_comb_fill$rel_score <- x_dim * (dff_mat_comb_fill$score / 400)
for_v_abline <- x_dim * (c(100,200,300,400) / 400)

#add the legend in colvec_legend_temp
par(xpd=TRUE)
legend(-1, nrow(any_mat)-1.75, legend=rep("", length(colvec_legend_temp)), col=colvec_legend_temp, pch=rep(15,length(colvec_legend_temp)) , bty="n", horiz=TRUE, x.intersp = -0.5, cex=1.5)
mtext("P-value" , side=1, line=-0.1, cex=0.8)
text(1, par("usr")[3] + 0.2, labels=seq(0, 0.1, length=3))


screen(2)
par(mar=c(1,0,6,0))
plot(1:x_dim, 1:x_dim, pch="", ylim=c(nrow(any_mat),2), xlim=c(0, nrow(any_mat)+2.5), axes=F, xlab="", ylab="")

for (i in 1:length(for_v_abline)) {
	segments(for_v_abline[i], c(0, x_dim+0.5 ,0, x_dim+0.5)[i], for_v_abline[i], 
	c(x_dim+0.5, 0, x_dim+0.5, 0)[i], lwd= grid_lwd, col="darkgrey")
}
for (j in 1:nrow(dff_mat_comb_fill)) {	
	x_start <- dff_mat_comb_fill$x_start[j]
	x_end <- dff_mat_comb_fill$rel_score[j]
	temp_col <- dff_mat_comb_fill$colors[j]
	lines(c(x_start, x_end),c(j,j), lwd=8.5, col=temp_col, lend=1)
}
text(c(0,for_v_abline), par("usr")[3] -0.5, labels=seq(0, 400, by=100), cex=0.7)
pos_lab <- mean(c(0,for_v_abline))
mtext("#of ions explained", cex=0.7, side=3, line=0)
text(pos_lab, par("usr")[1] +14.5, labels="#of substrates explained",cex=0.7)
text(pos_lab, par("usr")[1] +27.5, labels="#of products explained", cex=0.7)

dev.off()


#--------------------------------------
#for plotting data requirements
any_mat_df <- as.data.frame(any_mat) 
colnames(any_mat_df) <- names_cats
#write.csv(any_mat_df, "any_mat_df.csv", row.names=F)

#-----------------------------------------------
#writing monocol files to R list

monocol_ions_list <- list(monocols_ids_list, monocols_ids_list_substrate, monocols_ids_list_product)
names(monocol_ions_list) <- c("all", "substrate", "product")
lapply(monocol_ions_list, FUN=names)
save(monocol_ions_list, file="monocol_ions_list")
load("monocol_ions_list")

#-----------------------------------------------------------------------------------------------------
#prepare pollen ions to compare with
#used this pollen file to match the in vitro experiments. 

temp_ind_subs_ind <- ions_expl$idx[ions_expl$group == "microbial substrate"]
pollen_substrate_ids <- temp_ind_subs_ind[temp_ind_subs_ind %in% pollen_ion_ids_season]

write.table(ions[as.character(pollen_substrate_ids),], "ions in pollen and substrate group 20170302.tsv", row.names=F, quote=FALSE, sep='\t')

#-----------------------------------------------------------------------------------------------------
#now run the analyses in script "Kesnerova_et_al_R_in_vitro.R"
#-----------------------------------------------------------------------------------------------------

#matching the in vitro with in vivo patterns, considering pollen ions only! Further below considering all substrates

load("cond_pollen_inVitro_list") ##in vitro pollen substrates formulas
cond_pollen_inVitro_subs_list <- cond_pollen_inVitro_list
load("cond_pollen_inVitro_prod_list") #in vitro products formulas

#add the ionIdsx for the in vitro lists and use
names(monocols_ids_list_product)
names(monocols_ids_list_substrate)

#substrates
cond_pollen_inVitro_subs_list_ids <-  list()
cond_pollen_inVitro_subs_list_names <-  list()
for (i in 1:length(cond_pollen_inVitro_subs_list)) {
	cond_pollen_inVitro_subs_list_ids[[i]] <- which(ions$'Top annotation formula' %in% cond_pollen_inVitro_subs_list[[i]])
	cond_pollen_inVitro_subs_list_names[[i]] <- 
	ions$'Top annotation name'[cond_pollen_inVitro_subs_list_ids[[i]]]
}	
unlist(lapply(cond_pollen_inVitro_subs_list, FUN=length)) - unlist(lapply(cond_pollen_inVitro_subs_list_ids, FUN=length)) #all 0 because all are matched
names(cond_pollen_inVitro_subs_list_ids) <- names(cond_pollen_inVitro_subs_list)
names(cond_pollen_inVitro_subs_list_names) <- names(cond_pollen_inVitro_subs_list)


names(monocols_ids_list_substrate) #all explained substrates
names(cond_pollen_inVitro_subs_list_ids) #pollen ions observed in vitro


monocols_ids_list_substrate_overlap <- vector("list",length(monocols_ids_list_substrate))
monocols_ids_list_substrate_overlap_names <- vector("list",length(monocols_ids_list_substrate))
for (i in 1:length(monocols_ids_list_substrate)) {
	index <- which(names(cond_pollen_inVitro_subs_list_ids)  == names(monocols_ids_list_substrate)[i])
	if (length(index) >= 1) {
	monocols_ids_list_substrate_overlap[[i]] <- 
	monocols_ids_list_substrate[[i]][monocols_ids_list_substrate[[i]] %in%
	cond_pollen_inVitro_subs_list_ids[[index]]]
	monocols_ids_list_substrate_overlap_names[[i]] <- 
	ions$'Top annotation name'[monocols_ids_list_substrate_overlap[[i]]]
	}
}

lapply(monocols_ids_list_substrate_overlap, FUN=length)
names(monocols_ids_list_substrate_overlap) <- names(monocols_ids_list_substrate)
names(monocols_ids_list_substrate_overlap_names) <- names(monocols_ids_list_substrate)

#start with
length(unique(unlist(cond_pollen_inVitro_subs_list_ids))) #17

#in vivo and in vitro
length(unique(unlist(monocols_ids_list_substrate_overlap))) #13

overlap_in_vivo_in_vitro_substrate <- as.data.frame(cbind(
rep(names(monocols_ids_list_substrate_overlap), lapply(monocols_ids_list_substrate_overlap, FUN=length)),
unlist(monocols_ids_list_substrate_overlap),
unlist(monocols_ids_list_substrate_overlap_names)))
names(overlap_in_vivo_in_vitro_substrate) <- c("strain", "ionIdx", "name")

#three flavonoid-like compounds for F5
write.csv(overlap_in_vivo_in_vitro_substrate, "overlap_in_vivo_in_vitro_substratesv2.csv", row.names=F)
#--------------------------------------------
#matching the in vitro with in vivo patterns, considering all substrates

load("cond_allsubs_inVitro_list") ##in vitro pollen substrates formulas
cond_all_inVitro_subs_list <- cond_allsubs_inVitro_list

#all substrates
cond_all_inVitro_subs_list_ids <-  list()
cond_all_inVitro_subs_list_names <-  list()
for (i in 1:length(cond_all_inVitro_subs_list)) {
	cond_all_inVitro_subs_list_ids[[i]] <- which(ions$'Top annotation formula' %in% cond_all_inVitro_subs_list[[i]])
	cond_all_inVitro_subs_list_names[[i]] <- 
	ions$'Top annotation name'[cond_all_inVitro_subs_list_ids[[i]]]
}	

unlist(lapply(cond_all_inVitro_subs_list, FUN=length)) - unlist(lapply(cond_all_inVitro_subs_list_ids, FUN=length)) #not all are matched because of differential annotation between the two datasets
names(cond_all_inVitro_subs_list_ids) <- names(cond_all_inVitro_subs_list)
names(cond_all_inVitro_subs_list_names) <- names(cond_all_inVitro_subs_list)

names(monocols_ids_list_substrate) #all explained substrates
names(cond_all_inVitro_subs_list_ids) #all ions observed in vitro


monocols_ids_list_all_substrate_overlap <- vector("list",length(monocols_ids_list_substrate))
monocols_ids_list_all_substrate_overlap_names <- vector("list",length(monocols_ids_list_substrate))
for (i in 1:length(monocols_ids_list_substrate)) {
	index <- which(names(cond_all_inVitro_subs_list_ids)  == names(monocols_ids_list_substrate)[i])
	if (length(index) >= 1) {
	monocols_ids_list_all_substrate_overlap[[i]] <- 
	monocols_ids_list_substrate[[i]][monocols_ids_list_substrate[[i]] %in%
	cond_all_inVitro_subs_list_ids[[index]]]
	monocols_ids_list_all_substrate_overlap_names[[i]] <- 
	ions$'Top annotation name'[monocols_ids_list_all_substrate_overlap[[i]]]
	}
}
lapply(monocols_ids_list_all_substrate_overlap, FUN=length)
names(monocols_ids_list_all_substrate_overlap) <- names(monocols_ids_list_substrate)
names(monocols_ids_list_all_substrate_overlap_names) <- names(monocols_ids_list_substrate)

#start with
length(unique(unlist(cond_all_inVitro_subs_list_ids))) #73

#in vivo and in vitro
length(unique(unlist(monocols_ids_list_all_substrate_overlap))) #17

overlap_in_vivo_in_vitro_all_substrate <- as.data.frame(cbind(
rep(names(monocols_ids_list_all_substrate_overlap), lapply(monocols_ids_list_all_substrate_overlap, FUN=length)),
unlist(monocols_ids_list_all_substrate_overlap),
unlist(monocols_ids_list_all_substrate_overlap_names)))
names(overlap_in_vivo_in_vitro_all_substrate) <- c("strain", "ionIdx", "name")

#three flavonoid-like compounds for F5
write.csv(overlap_in_vivo_in_vitro_all_substrate, "overlap_in_vivo_in_vitro_all_substrates.csv", row.names=F)

#--------------------------------------------
#products
#this contains all groups, also other media and without pollen extract

cond_pollen_inVitro_prod_list_ids <-  list()
for (i in 1:length(cond_pollen_inVitro_prod_list)) {
	cond_pollen_inVitro_prod_list_ids[[i]] <- which(ions$'Top annotation formula' %in% cond_pollen_inVitro_prod_list[[i]])
}	
unlist(lapply(cond_pollen_inVitro_prod_list, FUN=length)) - unlist(lapply(cond_pollen_inVitro_prod_list_ids, FUN=length)) #not all matched
names(cond_pollen_inVitro_prod_list_ids) <- names(cond_pollen_inVitro_prod_list)

#only pollen extract ones
cond_pollen_inVitro_prod_list_ids_pe <- cond_pollen_inVitro_prod_list_ids[grep("PE", names(cond_pollen_inVitro_prod_list_ids))]
names(cond_pollen_inVitro_prod_list_ids_pe) <- gsub(" PE", "", names(cond_pollen_inVitro_prod_list_ids_pe))


monocols_ids_list_product_overlap <- vector("list",length(monocols_ids_list_product))
monocols_ids_list_product_overlap_names <- vector("list",length(monocols_ids_list_product))
for (i in 1:length(monocols_ids_list_product)) {
	index <- which(names(cond_pollen_inVitro_prod_list_ids_pe)  == names(monocols_ids_list_product)[i])
	if (length(index) >= 1) {
	monocols_ids_list_product_overlap[[i]] <- 
	monocols_ids_list_product[[i]][monocols_ids_list_product[[i]] %in%
	cond_pollen_inVitro_prod_list_ids_pe[[index]]]
	monocols_ids_list_product_overlap_names[[i]] <- 
	ions$'Top annotation name'[monocols_ids_list_product_overlap[[i]]]
	}
}
lapply(monocols_ids_list_product_overlap, FUN=length)
names(monocols_ids_list_product_overlap) <- names(monocols_ids_list_product)
names(monocols_ids_list_product_overlap_names) <- names(monocols_ids_list_product)


overlap_in_vivo_in_vitro_prod <- as.data.frame(cbind(
rep(names(monocols_ids_list_product_overlap), lapply(monocols_ids_list_product_overlap, FUN=length)),
unlist(monocols_ids_list_product_overlap),
unlist(monocols_ids_list_product_overlap_names)))
names(overlap_in_vivo_in_vitro_prod) <- c("strain", "ionIdx", "name")

write.csv(overlap_in_vivo_in_vitro_prod, "overlap_in_vivo_in_vitro_products_v2.csv", row.names=F)
#---------------------------------------------------------------------------------------

#now you can go back to the in vitro file

#---------------------------------------------------------------------------------------
#this again iterates over all ions to find the explained ones (= significantly different from MF) by any monocolonization
#S order is now corrected above
#finds possible cross feeding candidates among the substrates. Which ions are significantly higher from MF in a monocol, and also significantly lower in one or more monocols?


ion_all_id_explained_list <- vector("list", length=nrow(ions))
ion_id_all_group_list <- vector("list", length=nrow(ions))
ion_all_mf_dif_list <- vector("list", length=nrow(ions))
ion_all_mf_pval_list <- vector("list", length=nrow(ions))
ion_id_all_group_list_direction <- vector("list", length=nrow(ions))
cross_feeding_list_consumers <- vector("list", length=nrow(ions))
cross_feeding_list_donors <- vector("list", length=nrow(ions))


k=0
for (i in 1:length(groups_list_ion_aov_mf)) {
	ion_id <- i  
	data_stats_mf <- groups_list_ion_aov_mf[[ion_id]]
	data_z_score <- groups_list_ion_zcore[[ion_id]]
	significant_ones <- which(as.numeric(data_stats_mf$p.adj) < 0.05)
	significant_ones <- setdiff(significant_ones, 9)
	#3 is CL and 9 is MF, 10 is S and is corrected reversed comparison
	if (length(significant_ones) > 0) {
  		k <- k+1
  		ion_all_id_explained_list[[i]] <- i
  		ion_id_all_group_list[[i]] <- data_stats_mf$names[significant_ones]
  		z_score_diffs_cor <- -as.numeric(data_stats_mf$diff)
	  	if (z_score_diffs_cor[3] > 0) {
	  		ion_id_all_group_list_direction[[i]] <- 
	  		data_stats_mf$names[significant_ones[z_score_diffs_cor[significant_ones] 
	  		> 0]]
	  	} else {
	  		ion_id_all_group_list_direction[[i]] <- 
	  		data_stats_mf$names[significant_ones[z_score_diffs_cor[significant_ones] 
	  		< 0]] }	  	
  		
  		ion_all_mf_dif_list[[i]] <- z_score_diffs_cor[significant_ones]
  		ion_all_mf_pval_list[[i]] <- data_stats_mf$p.adj[significant_ones]
  		
  		#do significant ones have both directions compared to MF?
  		significant_ones_monocol <- setdiff(significant_ones, 8)
  		smaller_MF <- significant_ones_monocol[which(z_score_diffs_cor
  		[significant_ones_monocol] < z_score_diffs_cor[9])]
  		larger_MF <- significant_ones_monocol[which(z_score_diffs_cor
  		[significant_ones_monocol] > z_score_diffs_cor[9])]
  		if (length(smaller_MF) > 1 &  length(larger_MF) > 1) {
  			cross_feeding_list_consumers[[i]] <- 
  			paste(data_stats_mf$names[smaller_MF], collapse=";") 
  			cross_feeding_list_donors[[i]] <- 
  			paste(data_stats_mf$names[larger_MF], collapse=";") 
  		}
  	}	
}
#
#table(unlist(ion_id_all_group_list_direction)) #only same dir
#table(unlist(ion_id_all_group_list)) #both dir

k
temp_rows <- unlist(ion_all_id_explained_list)
head(ion_id_all_group_list)
head(ion_all_mf_dif_list)
head(ion_all_mf_pval_list)

ions$sign_vs_MF <- unlist(lapply(ion_id_all_group_list, FUN=paste, collapse=";"))
ions$sign_vs_MF_z_score_diff <- unlist(lapply(ion_all_mf_dif_list, FUN=paste, collapse=";"))
ions$sign_vs_MF_anova_pval <- unlist(lapply(ion_all_mf_pval_list, FUN=paste, collapse=";"))

dim(ions)
#-------------------------------------------
#looking furhter into the possibly cross fed ions predicted from FIA data

ion_names <- c()
ion_names_temp <- strsplit(ions$'Top annotation name', ";", fixed=TRUE)
ion_names <- do.call(rbind, ion_names_temp)
ion_names_first <- ion_names[,1]
ion_nr_ann <- unlist(lapply(strsplit(ions$'Top annotation ids', ";", fixed=TRUE), FUN=length))
ion_names_first[ion_nr_ann > 1] <- paste(ion_names_first[ion_nr_ann > 1], "*", sep="")

ions_cross_fed <- which(lapply(cross_feeding_list_consumers, FUN=nchar) > 0)
length(ions_cross_fed) #26
length(which(ions_cross_fed %in% sig_ions_seas_ind)) #19 / 26
cross_fed_all <- cbind(ions_cross_fed , ion_names_first[ions_cross_fed], as.character(cross_feeding_list_consumers[ions_cross_fed]), as.character(cross_feeding_list_donors[ions_cross_fed]), ion_nr_ann[ions_cross_fed])
cross_fed_all <- as.data.frame(cross_fed_all)
names(cross_fed_all) <- c("ionIdx", "name", "donors", "consumers","nr_ann")
write.table(cross_fed_all, "cross fed all.tsv", row.names=F, quote=FALSE, sep='\t')

#---------------------------------------------------------------------------------------
#overlapping the targeted with the untargeted data
#17 quantified, 12 of which also annotated in FIA data

#annotated from: interpolated_mm_gram_comparison_df$metabolite
# [1] "arginine"      "asparagine"    "glutamine"     "hexoses"      
# [5] "disaccharides" "tyrosine"      "adenine"       "phenylalanine"
# [9] "aspartate"     "glutamate"     "glucuronate"   "lactate"      
#[13] "tryptophane"   "pyruvate"      "succinate"     "malate"       
#[17] "fumarate"     

#12 annotated in both, correlation in fold change 0.77
# "arginine"      "asparagine"    "glutamine"   "tyrosine"        
# "phenylalanine" "aspartate"     "glutamate"     "glucuronate"  
# "pyruvate"      "succinate"     "malate"       "fumarate"     

addErrorBars <- function( x, y, err, ... ){ 	arrows( x, y+err, x, y-err, angle=90, code=3, length=.02, ...) }
addErrorBars_horz <- function( x, y, err, ... ){ 	arrows( x+err, y, x-err, y, angle=90, code=3, length=.02, ...) }


load("interpolated_mm_gram_comparison_df") #df made in the quantitative part code

temp_list_row <- list()
temp_list_name <- list()
for (i in 1:nrow(interpolated_mm_gram_comparison_df)) {
	temp_list_row[[i]] <- grep(interpolated_mm_gram_comparison_df$metabolite[i], tolower(ions$'Top annotation name'))
	temp_list_name[[i]] <- ions$'Top annotation name'[temp_list_row[[i]]]	
}
names(temp_list_row) <- interpolated_mm_gram_comparison_df$metabolite
names(temp_list_name) <- interpolated_mm_gram_comparison_df$metabolite
interpolated_mm_gram_comparison_df$list_pos <- c(1,1,1,NA,NA,1,NA,1,2,1,1,NA,NA, 1, 2, 1, 1)

interpolated_mm_gram_comparison_df$ions_row <- "-"
interpolated_mm_gram_comparison_df$ions_name <- "-"
for (i in 1:nrow(interpolated_mm_gram_comparison_df)) {
	interpolated_mm_gram_comparison_df$ions_row[i] <- temp_list_row[[i]][interpolated_mm_gram_comparison_df$list_pos[i]]
	interpolated_mm_gram_comparison_df$ions_name[i] <- ions$'Top annotation name'[as.numeric(interpolated_mm_gram_comparison_df$ions_row[i])]
}

#compare fold changes MF CL 

interpolated_mm_gram_comparison_df$log2FC_MF_CL <- "-"
interpolated_mm_gram_comparison_df$se_log2 <- "-"
row_mf <- which(names(interpolated_mm_gram_comparison_df) == "MF")
row_cl <- which(names(interpolated_mm_gram_comparison_df) == "CL")
for (i in 1:nrow(interpolated_mm_gram_comparison_df)) {
	ctr_int <- as.numeric(interpolated_mm_gram_comparison_df[i, row_mf])
	test_int <- as.numeric(interpolated_mm_gram_comparison_df[i, row_cl])
	#need standard errors of both to calculate the fold change error
	x <- na.omit(log2(as.numeric(ctr_int)))
	x_se <- sqrt(var(x)/length(x))
	y <- na.omit(log2(as.numeric(test_int)))
	y_se <- sqrt(var(y)/length(y))
	interpolated_mm_gram_comparison_df$se_log2[i] <- sqrt(x_se^2+y_se^2)
	#set at 0 when negative numbers
	ctr_int[ctr_int < 0] <- 0
	test_int[test_int < 0] <- 0
	interpolated_mm_gram_comparison_df$log2FC_MF_CL[i] <- log2(mean(test_int) / mean(ctr_int))
}

#compare with fall ones, from the ttest_res_list$fall results

ions_fc <- na.omit(ttest_res_list$fall$log2[as.numeric(interpolated_mm_gram_comparison_df$ions_row)]) #5 omitted out of 17
james_fc <- interpolated_mm_gram_comparison_df$log2FC_MF_CL[-c(4, 5, 7, 12, 13)]

ions_fc_se <- na.omit(ttest_res_list$fall$se_log2[as.numeric(interpolated_mm_gram_comparison_df$ions_row)])
james_fc_se <- interpolated_mm_gram_comparison_df$se_log2[-c(4, 5, 7, 12, 13)]

cor(as.numeric(ions_fc), as.numeric(james_fc)) #0.77 R2
cor(as.numeric(ions_fc_se), as.numeric(james_fc_se)) #-0.28 between errors


#with se error bars

pdf("overlap untargeted and targeted v2.pdf", width=8, height=4)
par(mfrow=c(1,2))
par(mar=c(4,4,1,1))
plot(ions_fc, james_fc, pch="", xlab="Q-TOF log2(FC)", ylab="LC-MS/MS log2(FC)", ylim=c(-4,2.5), xlim=c(-4,2.5), las=1)
fit <- lm(as.numeric(james_fc) ~ as.numeric(ions_fc))
abline(fit, lwd=2, col="darkgrey")
text(as.numeric(ions_fc), as.numeric(james_fc), labels=interpolated_mm_gram_comparison_df$metabolite[-c(4, 5, 7, 12, 13)], cex=0.7)
text(1.5,-1.5, labels="R2=0.77", cex=1)

par(mar=c(4,4,1,1))
plot(ions_fc, james_fc, pch="", xlab="Q-TOF log2(FC)", ylab="LC-MS/MS log2(FC)", ylim=c(-4,2.5), xlim=c(-4,2.5), las=1)
fit <- lm(as.numeric(james_fc) ~ as.numeric(ions_fc))
abline(fit, lwd=2, col="darkgrey")
points(ions_fc, james_fc, pch=1)
addErrorBars(as.numeric(ions_fc), as.numeric(james_fc), as.numeric(james_fc_se))
addErrorBars_horz(as.numeric(ions_fc), as.numeric(james_fc), as.numeric(ions_fc_se))

dev.off()

#---------------------------------------------------------------------------------------#for supplementary figure 2
#these files have the log2_pooled and the standard error. The log2 pooled is from both but the se from only fall?

addErrorBars <- function( x, y, err, ... ){ 	arrows( x, y+err, x, y-err, angle=90, code=3, length=.02, ...) }

df_for_barplot1 <- read_excel("Nucleosides_suppl_fig2.xlsx", sheet=2) 
df_for_barplot2 <- read_excel("flavonoids_suppl_fig2.xlsx", sheet=2) 
df_for_barplot3 <- read_excel("PG_JH_suppl_fig2.xlsx", sheet=1) 
df_for_barplot4 <- read_excel("Cacids_Kacids_suppl_fig.xlsx", sheet=1) 
df_for_barplot5 <- read_excel("pollen_suppl_fig.xlsx", sheet=1) 

#list of ions idx to plot
#get the statistics from the ions file
#plot barplot
#two plots, same height, width dependent on number in ionIdx

df_list <- list(df_for_barplot1, df_for_barplot2, df_for_barplot3, df_for_barplot4, df_for_barplot5)
ion_idx_list <- list(df_for_barplot1$ionIdx, df_for_barplot2$ionIdx, df_for_barplot3$ionIdx, df_for_barplot4$ionIdx, df_for_barplot5$ionIdx)

lapply(df_list, FUN=names)
lapply(df_list, FUN=dim)
lapply(df_list, function(x) {print(x$group)})

#----------------------------------------------
#preparing names
ion_names <- c()
ion_names_first <- ions$curated_annotation_with_all
ion_nr_ann <- unlist(lapply(strsplit(ions$'Top annotation ids', ";", fixed=TRUE), FUN=length))
ion_names_first[ion_nr_ann > 1] <- paste(ion_names_first[ion_nr_ann > 1], "*", sep="")
#remove the * for afzelin, quercitrin, and rutin c(821, 835, 995)
ion_names_first[c(821, 835, 995)] <- gsub("\\*", "", ion_names_first[c(821, 835, 995)])

#add the m/z to the ion
mz_temp <- substr(ions$ionMz, 1, 7)
ion_first_names__mz <- paste(mz_temp, ion_names_first, sep="   ")
#----------------------------------------------

#adapted to do on all substrates
overlap_in_vivo_in_vitro_substrate  <-read.csv( "overlap_in_vivo_in_vitro_all_substrates.csv", stringsAsFactors=F)
overlap_in_vivo_in_vitro_prod <- read.csv("overlap_in_vivo_in_vitro_products_v2.csv", stringsAsFactors =F)
overlap_idx <- c(overlap_in_vivo_in_vitro_substrate$ionIdx, overlap_in_vivo_in_vitro_prod$ionIdx)
overlap_strain <- c(overlap_in_vivo_in_vitro_substrate$strain, overlap_in_vivo_in_vitro_prod$strain)
overlap_df <- as.data.frame(cbind(as.character(overlap_idx), as.character(overlap_strain)))
names(overlap_df) <- c("ionIdx", "strain")

excel_filename <- "bee microbiota metabolomics data in vitro.xlsx" 
ions_sup <- read_excel(excel_filename, sheet=1) 
ions_in_sup <- which(ions$'Top annotation formula' %in%  ions_sup$'Top annotation formula')
length(ions_in_sup) #426 / 1079, 39.5%

#----------------------------------------
#start of iterations, start of plot
#requires having ions$log2_pooled to be present

names(df_list) <- c("nucleosides", "flavonoids", "host compounds", "acids", "pollen")

frac <- 0.3 
m <- rbind(c(0, 1, frac, 1), c(0, 1, 0, frac))
dev.off()
split.screen(m)
dev.off()

for (n in 1:length(df_list)) {

ions_to_plot <- df_list[[n]]$ionIdx

ions_to_plot_length <- length(ions_to_plot)
ymin <- floor(min(ions$log2_pooled[ions_to_plot]))
ymax <- ceiling(max(ions$log2_pooled[ions_to_plot]))

if (ymax < 0) {  ymax <- 0 }

filename <- paste("supplement_v3_", names(df_list)[n], ".pdf", sep="")
width_corrected <- 4 * (length(ions_to_plot) / 35) + 1

pdf(filename, width= width_corrected, height=5.5)

split.screen(m)
screen(1)
par(mar=c(0,3,1,1))
plot(1:ions_to_plot_length, 1:ions_to_plot_length, pch="", ylim=c(ymin, ymax), xlim=c(-1, ions_to_plot_length), axes=F, xlab="", ylab="")
par(cex.axis=0.8)
axis(2, las=1)
mtext("log2(FC) CL:MF", side=2, line=2, cex=0.8)


num_of_colors <- length(unique(df_list[[n]]$group))
colors_unique <- brewer.pal(num_of_colors,"Dark2")
require(rafalib)
group <- as.fumeric(df_list[[n]]$group)
colvec <- rep(colors_unique)[group]

#add symbol when significant
to_add_sig_star <- ions_to_plot[ions_to_plot %in% which(ions$robust_changes == "x")]
for (j in 1:ions_to_plot_length) {	
	if (ions$log2_pooled[ions_to_plot[j]] >= 0) {
		x_start <- 0
		x_end <- ions$log2_pooled[ions_to_plot[j]]
	} else {
		x_start <- ions$log2_pooled[ions_to_plot[j]]
		x_end <- 0 
	}
	temp_col <- colvec[j] 
	lines(c(j,j), c(x_start, x_end), lwd=8.5, col=temp_col, lend=1)
	pos <- (abs(ymax) + abs(ymin)) / 25
	#determines on which side to draw the line
	if (ions$log2_pooled[ions_to_plot[j]] >= 0) {
		text(j+1,-pos, ion_first_names__mz[ions_to_plot[j]], srt=70, cex=0.5, pos=2)
		addErrorBars(j, x_end, ions$se_log2_pooled[ions_to_plot[j]])
		if (ions_to_plot[j] %in% to_add_sig_star) {
			text(j, x_end+ pos, "*") }
	} else {		
		text(j-1, pos, ion_first_names__mz[ions_to_plot[j]], srt=70, cex=0.5, pos=4)
		addErrorBars(j, x_start, ions$se_log2_pooled[ions_to_plot[j]])
		if (ions_to_plot[j] %in% to_add_sig_star) {
			text(j, x_start-pos, "*") }
	}
}
names_cats <- unique(df_list[[n]]$group)
names_cats <- gsub("and Analogues", "& An.",names_cats)
names_cats <- gsub("and Derivatives", "& Der.",names_cats)
legend(0, ymin+0.5, legend=names_cats, col=colors_unique[unique(group)], bty="n", cex=0.7, pch=15, x.intersp = 1)


screen(2)
par(mar=c(1,3,1,1))

temp_explained_group_list <- ion_id_all_group_list_direction[ions_to_plot]
mat_one_expl <- matrix(data=0, ncol= ions_to_plot_length, nrow=length(groups))
groups_sort <- groups[c(3,1:2,4:8,10,9)]

for (i in 1:length(temp_explained_group_list)) {
	rows_mat_temp <- which(groups_sort %in% temp_explained_group_list[[i]])
	mat_one_expl[rows_mat_temp,i] <- 1
}
mat_one_expl <- mat_one_expl[rev(1:nrow(mat_one_expl)), ]
plot(1:ions_to_plot_length, ions_to_plot_length:1, pch="", ylim= c(0,nrow(mat_one_expl)), xlim=c(-1, ions_to_plot_length), axes=F, xlab="", ylab="")


#in vitro annotated ones, grey shading
to_plot_box <- which(ions_to_plot %in% ions_in_sup)
k=1
for (p in 1:ncol(mat_one_expl)) {
	if (p %in% to_plot_box) {
	lines(c(to_plot_box[k], to_plot_box[k]), c(0.7,nrow(mat_one_expl)+0.5), lwd=8.5, col="darkgrey", lend=1)
	k <- k+1
	}
}

for (j in 1:nrow(mat_one_expl)) {
	for (p in 1:ncol(mat_one_expl)) {
		if (mat_one_expl[j,p] == 1) {
		points(p, j, pch=15, col= "black")
		}		
	}
	text(0.5, j, rev(groups_sort)[j], cex=0.7, pos=2) 
}

overlap_test_rows <- which(as.character(overlap_df$ionIdx) %in% ions_to_plot)
#points indicating explained in same direction
if (length(overlap_test_rows) > 0) {
	for (z in 1:length(overlap_test_rows)) {
	overlap_strain_temp <- as.character(overlap_df$strain)[overlap_test_rows[z]]
	overlap_test_row <- which(rev(groups_sort) == overlap_strain_temp)
	overlap_test_col <- which(ions_to_plot == as.character(overlap_df$ionIdx)[overlap_test_rows[z]])
	text(overlap_test_col, overlap_test_row, labels="*", cex=1, col="white")	
	}
}
dev.off()
}

#end of plot
