##OPLSDA plot, figure 1

#---------------------------------------------------------------------------------------
#OPLS-DA plots. Use log10 int MF CL gut fall and spring together
#oplda part, plot it on the fall data

source("http://bioconductor.org/biocLite.R")
biocLite("ropls")
require(ropls)

ind <- grep("MF|CL", injections_mean$full_sample_group)
intensities_mean_sel <- intensities_mean[,ind] 
injections_mean_sel <- injections_mean[ind,]
names(intensities_mean_sel) <- injections_mean$full_sample_group[ind]
table(names(intensities_mean_sel))

#only on fall
intensities_mean_sel_gut <- intensities_mean_sel[,injections_mean$season[ind] == "fall"]
colnames(intensities_mean_sel_gut) <- injections_mean_sel$group[injections_mean$season[ind] == "fall"]

oplsda <- opls(t(intensities_mean_sel_gut), colnames(intensities_mean_sel_gut),  predI = 1, orthoI = NA) 

list_cor <- list()
list_cov <- list()
for (j in 1:nrow(intensities_mean_sel_gut)) {
	list_cor[[j]] <- cor(log10(as.numeric(intensities_mean_sel_gut[j,])), oplsda@scoreMN)
	list_cov[[j]] <- cov(log10(as.numeric(intensities_mean_sel_gut[j,])), oplsda@scoreMN)
}
#hist(unlist(list_cor))
#hist(unlist(list_cov))
#-----------------------------------------
#for the Plos Biology data requirements

#list_cor, list_cov, log2 FC

data_fig3_main <- as.data.frame(cbind(unlist(list_cor), unlist(list_cov), ions$log2_fall), stringsAsFactors=F)

head(data_fig3_main)
names(data_fig3_main) <-  c("oplsda_correlation", "oplsda_covariance", "log2_FC_fall")

data_fig3_inset <- as.data.frame(cbind(as.numeric(oplsda@scoreMN), as.numeric(oplsda@orthoScoreMN)))
names(data_fig3_inset) <- c("oplsda@scoreMN_Comp_1", "oplsda@orthoScoreMN")
data_fig3_inset$group <- row.names(oplsda@scoreMN)

#write.csv(data_fig3_main, "data_fig3_main.csv", row.names=F)
#write.csv(data_fig3_inset, "data_fig3_inset.csv", row.names=F)

#-----------------------------------------
#start of oplsda plot with annotations

#cutoffs correlation and covariance (abs)
gut_sig_ions <- sig_ions_seas_ind

require(RColorBrewer)

#substrates
gut_sig_ions_substr <- gut_sig_ions[which(unlist(list_cov)[gut_sig_ions] >= 5 & unlist(list_cor)[gut_sig_ions] >= 0.8)]
ions$'Top annotation name'[gut_sig_ions_substr] #42, fall only 26 with vignettes
gut_sig_ions_all_substr <- gut_sig_ions[which(unlist(list_cov)[gut_sig_ions] >= 0.1)]

#products
gut_sig_ions_prod <- gut_sig_ions[which(unlist(list_cov)[gut_sig_ions] <= -5 & unlist(list_cor)[gut_sig_ions] <= -0.8)]
ions$'Top annotation name'[gut_sig_ions_prod] #25, fall only 23 with vignettes
gut_sig_ions_all_prod <- gut_sig_ions[which(unlist(list_cov)[gut_sig_ions] <= -0.1)]

#-----------------------------------------
#updated names
#get the first name of the annotation $ number of alternative annotations

ion_names <- c()
#ion_names_temp <- strsplit(ions$'Top annotation name', ";", fixed=TRUE)
#ion_names <- do.call(rbind, ion_names_temp)
#ion_names_first <- ion_names[,1]
ion_names_first <- ions$curated_annotation_sign
ion_nr_ann <- unlist(lapply(strsplit(ions$'Top annotation ids', ";", fixed=TRUE), FUN=length))
ion_names_first[ion_nr_ann > 1] <- paste(ion_names_first[ion_nr_ann > 1], "*", sep="")
#remove the * for afzelin, quercitrin, and rutin c(821, 835, 995)
ion_names_first[c(821, 835, 995)] <- gsub("\\*", "", ion_names_first[c(821, 835, 995)])
#-----------------------------------------

#and make gut_sig_ions_ann
selrows <- list()
for (i in 1:length(gut_sig_ions)) {
  rows <- which(annotation$ionIdx == gut_sig_ions[i])
  selrows[[i]] <- rows[1]
}
gut_sig_ions_ann <- unlist(selrows)

table(annotation$detailed_compound_class[gut_sig_ions_ann]) #187 / 372, 49.8% has a compound class

m <- rbind(c(0, 0.25, 0, 1), c(0.25, 0.75, 0, 1), c(0.75, 1, 0, 1))
dev.off() #sometimes necessary to initiate grid properly
split.screen(m)
dev.off()


#start of plot. Note, some names and legends were updated later in illustrator.

pdf("oplsda fall only v5.pdf", width=9.5, height=5)

split.screen(m)
require(grid)

#left side
grid.segments(x0 = 0.25, y0 = 0.52, x1 = 0.34, y1 = 0.28, gp=gpar(col="darkgrey", lty="dashed"))
grid.segments(x0 = 0.25, y0 = 0.01, x1 = 0.34, y1 = 0.17, gp=gpar(col="darkgrey", lty="dashed"))

#right side
grid.segments(x0 = 0.733, y0 = 0.87, x1 = 0.758, y1 = 0.865, gp=gpar(col="darkgrey", lty="dashed"))
grid.segments(x0 = 0.733, y0 = 0.77, x1 = 0.758, y1 = 0.235, gp=gpar(col="darkgrey", lty="dashed"))

#mu <- 2.8
#plot(1:3, main=bquote(mu == .(mu)))

screen(1)
#left are the products, empty plot with legend 
gut_sig_ions_prod <- gut_sig_ions[which(unlist(list_cov)[gut_sig_ions] <= -5 & unlist(list_cor)[gut_sig_ions] <= -0.8)]
gut_sig_ions_prod_rank <- which(unlist(list_cov)[gut_sig_ions] <= -5 & unlist(list_cor)[gut_sig_ions] <= -0.8)
#sorting
sorted_temp <- order(unlist(list_cov)[gut_sig_ions_prod])
sorted_names_prod <- ion_names_first[gut_sig_ions_prod[sorted_temp]]
sorted_names_prod <- gsub("alpha", "a", sorted_names_prod)
sorted_names_prod <- gsub("beta", "b", sorted_names_prod)
#add the m/z to the ion
sorted_mz_prod <- substr(ions$ionMz[gut_sig_ions_prod[sorted_temp]], 1, 7)
sorted_names_prod_mz <- paste(sorted_mz_prod, sorted_names_prod, sep="   ")

sorted_cats_prod <- annotation$detailed_compound_class[gut_sig_ions_ann[gut_sig_ions_prod_rank[sorted_temp]]]
#add colors to the cats
gut_groups <- c("Fatty Acids and Conjugates", "Amino Acids and Derivatives", "Carboxylic Acids and Derivatives", "Flavonoids", "Eicosanoids", "Pyrimidine Nucleosides and Analogues", "Purine Nucleosides and Analogues")
colors_unique <- brewer.pal(length(gut_groups),"Dark2")[c(6,2,4,3,1,7)]
colors_unique_plus <- c(colors_unique, colors_unique[length(colors_unique)]) #repeat the last one
colors_cats <- rep("black", times=length(sorted_cats_prod))
for (i in 1:length(sorted_cats_prod)) {
	ind_match <- which(gut_groups == sorted_cats_prod[i])
	if (length(ind_match) > 0) {
		colors_cats[i] <- colors_unique_plus[ind_match]
	}
}
par(mar=c(0,0,0,0))
plot(1:10,1:10, axes=F, pch="")
comb_title <- paste("Most discriminatory products", "\n", "  m/z", "    curated name")
legend(1,6, sorted_names_prod_mz, text.col=colors_cats , bty="n", cex=0.55, title= comb_title, title.adj=0.08, title.col="black")
polygon(c(1.5,10.3,10.3,1.5), c(0.7,0.7,5.7,5.7), lty="dashed", border="darkgrey")


screen(2)

basic_cex <- 0.5
sign_cex <- (log10(abs(ions$log2_fall[gut_sig_ions])+1) * 2) + 0.4
sign_cex_all <- (log10(abs(ions$log2_fall)+1) * 2) + 0.4
#5 is the max, prepare for legend
legend_cex <- log10(c(0:4+1)) * 2 + 0.4
legend_cex_text <- c(0:4)


par(mar=c(4,4,3,0.5))
plot(unlist(list_cov), unlist(list_cor), xlim=c(-12,12), ylim=c(-1, 1), xlab="", ylab="", pch="", las=1)
mtext("Covariance ion int. with Component 1", side=1, line=2.2)
mtext("Correlation ion int. with Component 1", side=2, line=2.7)
abline(h=0, col="darkgrey")
abline(v=0, col="darkgrey")
points(unlist(list_cov), unlist(list_cor), pch=16, cex=basic_cex, col=rgb(0.6, 0.6, 0.6, 0.5))
points(unlist(list_cov)[gut_sig_ions], unlist(list_cor)[gut_sig_ions], cex=sign_cex, pch=16, col="black")

#put a box around the selected ions
polygon(c(5,12.5,12.5,5), c(0.75,0.75,1.05,1.05), lty="dashed", border="darkgrey")
polygon(c(-5,-12.5,-12.5,-5), c(-0.75,-0.75,-1.05,-1.05), lty="dashed", border="darkgrey")

for (i in 1:length(gut_groups)) {
    rows <- gut_sig_ions[grep(gut_groups[i], 
    	annotation$detailed_compound_class[gut_sig_ions_ann])]
    points(unlist(list_cov)[rows], unlist(list_cor)[rows], pch=16, cex=sign_cex_all[rows], col= colors_unique_plus[i])
}
gut_groups_names <- c("Fatty Acids & Conjug.", "Amino Acids & Deriv.", "Carboxylic Acids & Deriv.", "Flavonoids", "Eicosanoids", "Nucleosides & Analogs")

before_legend_name <-c("Not significant", "All significant")
before_legend_col <-c("darkgrey", "black")
legend(0.7, 0, legend= c(before_legend_name, gut_groups_names), pch=16, col= c(before_legend_col, colors_unique), cex=0.87, bty="n")
text(c(-2.5) , par("usr")[3] + 2.25, labels=expression("" %<-% "Higher in CL bees"), cex=1, xpd=TRUE, adj=1)
text(c(13.3) , par("usr")[3] + 2.25, labels=expression("Higher in MF bees" %->% ""), cex=1, xpd=TRUE, adj=1)

legend(-11.5, 0, legend=as.character(legend_cex_text) , pch=16, col= "black", pt.cex=legend_cex, bty="n", cex=0.87, title="abs(log2(FC))")


#inset plot
pchvec <- c(1:length(oplsda@scoreMN))
pchvec[grep("CL", rownames(oplsda@scoreMN))] <- 16
pchvec[grep("MF", rownames(oplsda@scoreMN))] <- 16
colvec <- c(1:length(oplsda@scoreMN))
colvec[grep("CL", rownames(oplsda@scoreMN))] <- "black"
colvec[grep("MF", rownames(oplsda@scoreMN))] <- rgb(0.6, 0.6, 0.6, 0.5)

par(fig=c(0.27, 0.47, 0.48, 0.95), new=T)
plot(oplsda@scoreMN, oplsda@orthoScoreMN, col=colvec, pch=pchvec, xlab="", ylab="", las=1, axes=F)
axis(side = 2, las=1, cex.axis=0.8, labels=F, lwd.ticks=0)
axis(side = 1, las=1, cex.axis=0.8, labels=F, lwd.ticks=0)
mtext("Component 1", side=1, cex=0.9)
mtext("Orthogonal comp.", side=2, cex=0.9)
mtext("O-PLS-DA", side=3, cex=0.9)
#legend(-14, -5, c("CL", "MF"), pch=unique(pchvec), col=rev(unique(colvec)), cex=0.8)
box()
text(c(-14) , par("usr")[3] + 15, labels=c("CL"), cex=1, xpd=TRUE, adj=1)
text(c(21) , par("usr")[3] + 15, labels=c("MF"), cex=1, xpd=TRUE, adj=1, col="darkgrey")

screen(3)
#right side are the substrates
gut_sig_ions_substr <- gut_sig_ions[which(unlist(list_cov)[gut_sig_ions] >= 5 & unlist(list_cor)[gut_sig_ions] >= 0.8)]
gut_sig_ions_substr_rank <- which(unlist(list_cov)[gut_sig_ions] >= 5 & unlist(list_cor)[gut_sig_ions] >= 0.8)

#sorting
sorted_temp <- rev(order(unlist(list_cov)[gut_sig_ions_substr]))
sorted_names_subst <- ion_names_first[gut_sig_ions_substr[sorted_temp]]
sorted_names_subst <- gsub("alpha", "a", sorted_names_subst)
sorted_names_subst <- gsub("beta", "b", sorted_names_subst)

#add the m/z to the ion
sorted_mz_substr <- substr(ions$ionMz[gut_sig_ions_substr[sorted_temp]], 1, 7)
sorted_names_subst_mz <- paste(sorted_mz_substr, sorted_names_subst, sep="   ")

sorted_cats_subst <- annotation$detailed_compound_class[gut_sig_ions_ann[gut_sig_ions_substr_rank[sorted_temp]]]
#add colors to the cats
colors_cats <- rep("black", times=length(sorted_names_subst))
for (i in 1:length(sorted_cats_subst)) {
	ind_match <- which(gut_groups == sorted_cats_subst[i])
	if (length(ind_match) > 0) {
		colors_cats[i] <- colors_unique_plus[ind_match]
	}
}
par(mar=c(0,0,0,0))
plot(1:10,1:10, axes=F, pch="")
comb_title <- paste("Most discriminatory substrates", "\n", "m/z", "    curated name")
legend(0.5,9.3, sorted_names_subst_mz, text.col=colors_cats , bty="n", cex=0.55, title= comb_title, title.adj=0.12, title.col="black")
polygon(c(1,10.3,10.3,1), c(2.9,2.9,9,9), lty="dashed", border="darkgrey")

dev.off()



#-----------------------------------------
#to locate specific ions, control+F on the pdf top annotation name

pdf("oplsda gut plot ion names.pdf", width=15, height=15) 
par(fig=c(0,1,0,1))
par(mar=c(4,4,3,1))
xmax <- ceiling(max(abs(unlist(list_cov))))
plot(unlist(list_cov), unlist(list_cor), xlim=c((xmax-2*xmax), xmax), ylim=c(-1, 1), xlab="Covariance intensity with Model score", ylab="Correlation intensity with Model score", pch="", las=1)
abline(h=0, col="darkgrey")
abline(v=0, col="darkgrey")
points(unlist(list_cov), unlist(list_cor), pch=16, cex=0.9, col=rgb(0.6, 0.6, 0.6, 0.5))
#points(unlist(list_cov)[gut_sig_ions], unlist(list_cor)[gut_sig_ions], cex=0.9, pch=16, col="black")

text(unlist(list_cov)[gut_sig_ions], unlist(list_cor)[gut_sig_ions], ion_names[gut_sig_ions,1], cex=0.5)
vignette_ids <- c(67,285,648,654,736,835,995)
text(unlist(list_cov)[gut_sig_ions], unlist(list_cor)[gut_sig_ions], gut_sig_ions,1, cex=0.5)
dev.off()





#---------------------------------------------------------------------------------------------------------------

#testing effect on leaving out a single one on the vignette ions, 1000 iterations

ind_list <- list()
gut_sig_ions_substr_list <- list()
gut_sig_ions_prod_list <- list()
for (i in 1:1000) {
	ind <- grep("MF|CL", injections_mean$full_sample_group)
	ind <- ind[-sample.int(length(ind), 1)]
	intensities_mean_sel <- intensities_mean[,ind] 
	injections_mean_sel <- injections_mean[ind,]
	names(intensities_mean_sel) <- injections_mean$full_sample_group[ind]
	#only on fall
	intensities_mean_sel_gut <- intensities_mean_sel[,injections_mean$season[ind] == "fall"]
	colnames(intensities_mean_sel_gut) <- injections_mean_sel$group[injections_mean$season[ind] == "fall"]
	oplsda <- opls(t(intensities_mean_sel_gut), colnames(intensities_mean_sel_gut),  predI = 1, orthoI = NA) 
	dev.off()

	list_cor <- list()
	list_cov <- list()
	for (j in 1:nrow(intensities_mean_sel_gut)) {
		list_cor[[j]] <- cor(log10(as.numeric(intensities_mean_sel_gut[j,])), oplsda@scoreMN)
		list_cov[[j]] <- cov(log10(as.numeric(intensities_mean_sel_gut[j,])), oplsda@scoreMN)
	}
	gut_sig_ions_substr <- gut_sig_ions[which(unlist(list_cov)[gut_sig_ions] >= 5 & unlist(list_cor)[gut_sig_ions] >= 0.8)]
#products
	gut_sig_ions_prod <- gut_sig_ions[which(unlist(list_cov)[gut_sig_ions] <= -5 & unlist(list_cor)[gut_sig_ions] <= -0.8)]

	ind_list[[i]] <- ind
	gut_sig_ions_substr_list[[i]] <- gut_sig_ions_substr
	gut_sig_ions_prod_list[[i]] <- gut_sig_ions_prod
}

#-----------------------------------------
#looking into leave one out strategy

table(unlist(ind_list))
table(unlist(gut_sig_ions_substr_list))
table(unlist(gut_sig_ions_prod_list))

#save(gut_sig_ions_substr, file="gut_sig_ions_substr")
#save(gut_sig_ions_prod, file="gut_sig_ions_prod")
load(file="gut_sig_ions_substr")
load(file="gut_sig_ions_prod")

gut_sig_ions_substr_perm <- gut_sig_ions_substr[gut_sig_ions_substr %in% names(which(table(unlist(gut_sig_ions_substr_list)) >= 1000))]
gut_sig_ions_prod_perm <- gut_sig_ions_prod[gut_sig_ions_prod %in% names(which(table(unlist(gut_sig_ions_prod_list)) >= 1000))]

ion_names_first[gut_sig_ions_substr_perm] #24
ion_names_first[gut_sig_ions_prod_perm] #19

ion_names_first[gut_sig_ions_substr] #28
ion_names_first[gut_sig_ions_prod] #23


ion_names_first[gut_sig_ions_substr][which(ion_names_first[gut_sig_ions_substr] %in% ion_names_first[gut_sig_ions_substr_perm] == FALSE)]
gut_sig_ions_substr[c(3, 12, 15, 18)] #166 496 626 687. Orotate 950
#[1] "Orotate*"        "Inosine"         "SC-57666"       
#[4] "Docosanoic acid"

ion_names_first[gut_sig_ions_prod][which(ion_names_first[gut_sig_ions_prod] %in% ion_names_first[gut_sig_ions_prod_perm] == FALSE)]
gut_sig_ions_prod[c(11, 15, 17, 20)] #262 396 507 778
#[1] "10-Oxodecanoate*"                                                          
#[2] "Dodecanedioic acid*"                                                       
#[3] "(10S)-Juvenile hormone III acid diol"                                      
#[4] "9alpha-Fluoro-11beta,16alpha,17alpha,21-tetrahydroxypregn-4-ene-3,20-dione"


